"""CoffeeShop DataFrame query implementations.

All 11 CoffeeShop benchmark queries implemented for both Expression and Pandas families.

Tables: order_lines (fact), dim_locations, dim_products (dimensions)
Patterns: joins, CASE WHEN, window functions, date operations, NULLIF

Query categories:
- Sales Analysis (SA1-SA5): Revenue, products, locations
- Product Analysis (PR1-PR2): Product mix, price bands
- Trend Analysis (TR1): Quarterly trends
- Time Analysis (TM1): Day-part cadence
- Quality Checks (QC1-QC2): Data quality metrics

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

from datetime import date
from typing import Any


def _parse_date(value: str | date) -> date:
    """Parse a date string (YYYY-MM-DD) to a date object, or return as-is if already a date."""
    if isinstance(value, date):
        return value
    return date.fromisoformat(value)


from benchbox.core.dataframe.context import DataFrameContext
from benchbox.core.dataframe.query import DataFrameQuery, QueryCategory

from .parameters import get_parameters
from .registry import register_query

# =============================================================================
# SA1: Daily revenue and order volume by region
# =============================================================================


def sa1_expression_impl(ctx: DataFrameContext) -> Any:
    """SA1: Daily revenue and order volume by region."""
    params = get_parameters("SA1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2023-01-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .join(dl, left_on="location_record_id", right_on="record_id")
        .group_by("order_date", "region")
        .agg(
            col("order_id").n_unique().alias("order_count"),
            col("total_price").sum().alias("gross_revenue"),
        )
        .with_columns(
            ctx.when(col("order_count") != lit(0))
            .then(col("gross_revenue") / col("order_count"))
            .otherwise(lit(None))
            .alias("avg_order_value")
        )
        .sort(["order_date", "region"])
    )


def sa1_pandas_impl(ctx: DataFrameContext) -> Any:
    """SA1: Daily revenue and order volume by region."""
    import numpy as np

    params = get_parameters("SA1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2023-01-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    merged = filtered.merge(dl, left_on="location_record_id", right_on="record_id")
    grouped = merged.groupby(["order_date", "region"], as_index=False).agg(
        order_count=("order_id", "nunique"),
        gross_revenue=("total_price", "sum"),
    )
    grouped["avg_order_value"] = np.where(
        grouped["order_count"] != 0, grouped["gross_revenue"] / grouped["order_count"], np.nan
    )
    return grouped.sort_values(["order_date", "region"])


# =============================================================================
# SA2: Top products by revenue for a given year
# =============================================================================


def sa2_expression_impl(ctx: DataFrameContext) -> Any:
    """SA2: Top products by revenue for a given year."""
    params = get_parameters("SA2")
    year = params.get("year", 2023)
    limit = params.get("limit", 15)

    ol = ctx.get_table("order_lines")
    dp = ctx.get_table("dim_products")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.with_columns(col("order_date").dt.year().alias("year"))
        .filter(col("year") == lit(year))
        .join(dp, left_on="product_record_id", right_on="record_id")
        .group_by("subcategory", "name")
        .agg(
            col("quantity").sum().alias("total_quantity"),
            col("total_price").sum().alias("total_revenue"),
        )
        .sort("total_revenue", descending=True)
        .limit(limit)
    )


def sa2_pandas_impl(ctx: DataFrameContext) -> Any:
    """SA2: Top products by revenue for a given year."""
    params = get_parameters("SA2")
    year = params.get("year", 2023)
    limit = params.get("limit", 15)

    ol = ctx.get_table("order_lines")
    dp = ctx.get_table("dim_products")
    filtered = ol[ol["order_date"].dt.year == year]
    merged = filtered.merge(dp, left_on="product_record_id", right_on="record_id")
    return (
        merged.groupby(["subcategory", "name"], as_index=False)
        .agg(total_quantity=("quantity", "sum"), total_revenue=("total_price", "sum"))
        .sort_values("total_revenue", ascending=False)
        .head(limit)
    )


# =============================================================================
# SA3: Monthly performance metrics
# =============================================================================


def sa3_expression_impl(ctx: DataFrameContext) -> Any:
    """SA3: Monthly performance metrics for a selected year."""
    params = get_parameters("SA3")
    year = params.get("year", 2023)

    ol = ctx.get_table("order_lines")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.with_columns(
            col("order_date").dt.year().alias("year"),
            col("order_date").dt.month().alias("month"),
        )
        .filter(col("year") == lit(year))
        .group_by("year", "month")
        .agg(
            col("order_id").n_unique().alias("orders"),
            col("quantity").sum().alias("items_sold"),
            col("total_price").sum().alias("revenue"),
        )
        .with_columns(
            ctx.when(col("orders") != lit(0))
            .then(col("revenue") / col("orders"))
            .otherwise(lit(None))
            .alias("avg_order_value"),
            ctx.when(col("orders") != lit(0))
            .then(col("items_sold") / col("orders"))
            .otherwise(lit(None))
            .alias("avg_items_per_order"),
        )
        .sort(["year", "month"])
    )


def sa3_pandas_impl(ctx: DataFrameContext) -> Any:
    """SA3: Monthly performance metrics for a selected year."""
    import numpy as np

    params = get_parameters("SA3")
    year = params.get("year", 2023)

    ol = ctx.get_table("order_lines")
    filtered = ol[ol["order_date"].dt.year == year].copy()
    filtered["year"] = filtered["order_date"].dt.year
    filtered["month"] = filtered["order_date"].dt.month
    grouped = filtered.groupby(["year", "month"], as_index=False).agg(
        orders=("order_id", "nunique"),
        items_sold=("quantity", "sum"),
        revenue=("total_price", "sum"),
    )
    grouped["avg_order_value"] = np.where(grouped["orders"] != 0, grouped["revenue"] / grouped["orders"], np.nan)
    grouped["avg_items_per_order"] = np.where(grouped["orders"] != 0, grouped["items_sold"] / grouped["orders"], np.nan)
    return grouped.sort_values(["year", "month"])


# =============================================================================
# SA4: Revenue share by region (window function)
# =============================================================================


def sa4_expression_impl(ctx: DataFrameContext) -> Any:
    """SA4: Revenue share by region using window function."""
    params = get_parameters("SA4")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    col = ctx.col
    lit = ctx.lit

    grouped = (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .join(dl, left_on="location_record_id", right_on="record_id")
        .group_by("region")
        .agg(
            col("order_id").n_unique().alias("orders"),
            col("total_price").sum().alias("revenue"),
        )
    )
    total_val = grouped.select(col("revenue").sum().alias("total")).scalar()
    return grouped.with_columns(
        ctx.when(lit(total_val) != lit(0))
        .then(col("revenue") / lit(total_val))
        .otherwise(lit(0.0))
        .alias("revenue_share")
    ).sort("revenue", descending=True)


def sa4_pandas_impl(ctx: DataFrameContext) -> Any:
    """SA4: Revenue share by region using window function."""
    params = get_parameters("SA4")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    merged = filtered.merge(dl, left_on="location_record_id", right_on="record_id")
    grouped = merged.groupby(["region"], as_index=False).agg(
        orders=("order_id", "nunique"),
        revenue=("total_price", "sum"),
    )
    total = grouped["revenue"].sum()
    grouped["revenue_share"] = grouped["revenue"] / total if total != 0 else 0
    return grouped.sort_values("revenue", ascending=False)


# =============================================================================
# SA5: Top-performing locations by revenue
# =============================================================================


def sa5_expression_impl(ctx: DataFrameContext) -> Any:
    """SA5: Top-performing locations by revenue."""
    params = get_parameters("SA5")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")
    limit = params.get("limit", 20)

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .join(dl, left_on="location_record_id", right_on="record_id")
        .group_by("location_id", "city", "state", "region")
        .agg(
            col("order_id").n_unique().alias("orders"),
            col("total_price").sum().alias("revenue"),
            col("total_price").mean().alias("avg_line_value"),
        )
        .sort("revenue", descending=True)
        .limit(limit)
    )


def sa5_pandas_impl(ctx: DataFrameContext) -> Any:
    """SA5: Top-performing locations by revenue."""
    params = get_parameters("SA5")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")
    limit = params.get("limit", 20)

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    merged = filtered.merge(dl, left_on="location_record_id", right_on="record_id")
    return (
        merged.groupby(["location_id", "city", "state", "region"], as_index=False)
        .agg(orders=("order_id", "nunique"), revenue=("total_price", "sum"), avg_line_value=("total_price", "mean"))
        .sort_values("revenue", ascending=False)
        .head(limit)
    )


# =============================================================================
# PR1: Product mix and revenue by subcategory
# =============================================================================


def pr1_expression_impl(ctx: DataFrameContext) -> Any:
    """PR1: Product mix and revenue by subcategory with date validity check."""
    params = get_parameters("PR1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2023-12-31")

    ol = ctx.get_table("order_lines")
    dp = ctx.get_table("dim_products")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .join(dp, left_on="product_record_id", right_on="record_id")
        .filter((col("order_date") >= col("from_date")) & (col("order_date") <= col("to_date")))
        .group_by("subcategory")
        .agg(
            col("product_id").n_unique().alias("active_products"),
            col("quantity").sum().alias("quantity_sold"),
            col("total_price").sum().alias("revenue"),
        )
        .sort("revenue", descending=True)
    )


def pr1_pandas_impl(ctx: DataFrameContext) -> Any:
    """PR1: Product mix and revenue by subcategory with date validity check."""
    params = get_parameters("PR1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2023-12-31")

    ol = ctx.get_table("order_lines")
    dp = ctx.get_table("dim_products")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    merged = filtered.merge(dp, left_on="product_record_id", right_on="record_id")
    merged = merged[(merged["order_date"] >= merged["from_date"]) & (merged["order_date"] <= merged["to_date"])]
    return (
        merged.groupby(["subcategory"], as_index=False)
        .agg(
            active_products=("product_id", "nunique"), quantity_sold=("quantity", "sum"), revenue=("total_price", "sum")
        )
        .sort_values("revenue", ascending=False)
    )


# =============================================================================
# PR2: Price-band distribution
# =============================================================================


def pr2_expression_impl(ctx: DataFrameContext) -> Any:
    """PR2: Price-band distribution using CASE WHEN."""
    params = get_parameters("PR2")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .with_columns(
            ctx.when(col("unit_price") < lit(4))
            .then(lit("Under $4"))
            .otherwise(
                ctx.when(col("unit_price") < lit(6))
                .then(lit("$4-$5.99"))
                .otherwise(ctx.when(col("unit_price") < lit(8)).then(lit("$6-$7.99")).otherwise(lit("$8+")))
            )
            .alias("price_band")
        )
        .group_by("price_band")
        .agg(
            col("price_band").count().alias("line_count"),
            col("total_price").sum().alias("revenue"),
        )
        .sort("price_band")
    )


def pr2_pandas_impl(ctx: DataFrameContext) -> Any:
    """PR2: Price-band distribution using CASE WHEN."""
    import numpy as np

    params = get_parameters("PR2")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)].copy()
    conditions = [
        filtered["unit_price"] < 4,
        filtered["unit_price"] < 6,
        filtered["unit_price"] < 8,
    ]
    choices = ["Under $4", "$4-$5.99", "$6-$7.99"]
    filtered["price_band"] = np.select(conditions, choices, default="$8+")
    return (
        filtered.groupby(["price_band"], as_index=False)
        .agg(line_count=("price_band", "count"), revenue=("total_price", "sum"))
        .sort_values("price_band")
    )


# =============================================================================
# TR1: Quarterly revenue and order growth
# =============================================================================


def tr1_expression_impl(ctx: DataFrameContext) -> Any:
    """TR1: Quarterly revenue and order growth."""
    params = get_parameters("TR1")
    start_year = params.get("start_year", 2023)
    end_year = params.get("end_year", 2024)

    ol = ctx.get_table("order_lines")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.with_columns(
            col("order_date").dt.year().alias("year"),
            col("order_date").dt.month().alias("month"),
        )
        .filter((col("year") >= lit(start_year)) & (col("year") <= lit(end_year)))
        .with_columns((((col("month") - lit(1)) / lit(3)).floor().cast_int() + lit(1)).alias("quarter"))
        .group_by("year", "quarter")
        .agg(
            col("order_id").n_unique().alias("orders"),
            col("total_price").sum().alias("revenue"),
        )
        .sort(["year", "quarter"])
    )


def tr1_pandas_impl(ctx: DataFrameContext) -> Any:
    """TR1: Quarterly revenue and order growth."""
    import numpy as np

    params = get_parameters("TR1")
    start_year = params.get("start_year", 2023)
    end_year = params.get("end_year", 2024)

    ol = ctx.get_table("order_lines")
    filtered = ol[(ol["order_date"].dt.year >= start_year) & (ol["order_date"].dt.year <= end_year)].copy()
    filtered["year"] = filtered["order_date"].dt.year
    filtered["quarter"] = np.floor((filtered["order_date"].dt.month - 1) / 3).astype(int) + 1
    return (
        filtered.groupby(["year", "quarter"], as_index=False)
        .agg(orders=("order_id", "nunique"), revenue=("total_price", "sum"))
        .sort_values(["year", "quarter"])
    )


# =============================================================================
# TM1: Order cadence by day-part for a region
# =============================================================================


def tm1_expression_impl(ctx: DataFrameContext) -> Any:
    """TM1: Order cadence by day-part for a selected region."""
    params = get_parameters("TM1")
    region = params.get("region", "South")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    col = ctx.col
    lit = ctx.lit

    joined = (
        ol.filter(
            (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
        )
        .join(dl, left_on="location_record_id", right_on="record_id")
        .filter(col("region") == lit(region))
    )
    with_hour = joined.with_columns(col("order_time").dt.hour().alias("hour"))
    with_part = with_hour.with_columns(
        ctx.when((col("hour") >= lit(5)) & (col("hour") <= lit(10)))
        .then(lit("Morning"))
        .otherwise(
            ctx.when((col("hour") >= lit(11)) & (col("hour") <= lit(14)))
            .then(lit("Midday"))
            .otherwise(
                ctx.when((col("hour") >= lit(15)) & (col("hour") <= lit(17)))
                .then(lit("Afternoon"))
                .otherwise(lit("Evening"))
            )
        )
        .alias("day_part")
    )
    return (
        with_part.group_by("day_part")
        .agg(
            col("order_id").n_unique().alias("orders"),
            col("total_price").sum().alias("revenue"),
            col("total_price").mean().alias("avg_line_value"),
        )
        .sort("day_part")
    )


def tm1_pandas_impl(ctx: DataFrameContext) -> Any:
    """TM1: Order cadence by day-part for a selected region."""
    import numpy as np

    params = get_parameters("TM1")
    region = params.get("region", "South")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    merged = filtered.merge(dl, left_on="location_record_id", right_on="record_id")
    merged = merged[merged["region"] == region].copy()
    merged["hour"] = merged["order_time"].dt.hour
    conditions = [
        (merged["hour"] >= 5) & (merged["hour"] <= 10),
        (merged["hour"] >= 11) & (merged["hour"] <= 14),
        (merged["hour"] >= 15) & (merged["hour"] <= 17),
    ]
    choices = ["Morning", "Midday", "Afternoon"]
    merged["day_part"] = np.select(conditions, choices, default="Evening")
    return (
        merged.groupby(["day_part"], as_index=False)
        .agg(orders=("order_id", "nunique"), revenue=("total_price", "sum"), avg_line_value=("total_price", "mean"))
        .sort_values("day_part")
    )


# =============================================================================
# QC1: Average lines per order
# =============================================================================


def qc1_expression_impl(ctx: DataFrameContext) -> Any:
    """QC1: Average lines and items per order."""
    params = get_parameters("QC1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    col = ctx.col
    lit = ctx.lit

    filtered = ol.filter(
        (col("order_date") >= lit(_parse_date(start_date))) & (col("order_date") <= lit(_parse_date(end_date)))
    )

    return filtered.select(
        ctx.when(col("order_id").n_unique() != lit(0))
        .then(col("order_id").count().cast_float() / col("order_id").n_unique().cast_float())
        .otherwise(lit(None))
        .alias("avg_lines_per_order"),
        ctx.when(col("order_id").n_unique() != lit(0))
        .then(col("quantity").sum().cast_float() / col("order_id").n_unique().cast_float())
        .otherwise(lit(None))
        .alias("avg_items_per_order"),
    )


def qc1_pandas_impl(ctx: DataFrameContext) -> Any:
    """QC1: Average lines and items per order."""
    import numpy as np
    import pandas as pd

    params = get_parameters("QC1")
    start_date = params.get("start_date", "2023-01-01")
    end_date = params.get("end_date", "2024-12-31")

    ol = ctx.get_table("order_lines")
    filtered = ol[(ol["order_date"] >= start_date) & (ol["order_date"] <= end_date)]
    total_lines = len(filtered)
    unique_orders = filtered["order_id"].nunique()
    total_items = filtered["quantity"].sum()
    return pd.DataFrame(
        {
            "avg_lines_per_order": [total_lines / unique_orders if unique_orders else np.nan],
            "avg_items_per_order": [total_items / unique_orders if unique_orders else np.nan],
        }
    )


# =============================================================================
# QC2: Seasonal revenue comparison across regions
# =============================================================================


def qc2_expression_impl(ctx: DataFrameContext) -> Any:
    """QC2: Seasonal revenue comparison across regions."""
    params = get_parameters("QC2")
    start_year = params.get("start_year", 2023)
    end_year = params.get("end_year", 2024)

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    col = ctx.col
    lit = ctx.lit

    return (
        ol.with_columns(
            col("order_date").dt.year().alias("year"),
            col("order_date").dt.month().alias("month"),
        )
        .filter((col("year") >= lit(start_year)) & (col("year") <= lit(end_year)))
        .join(dl, left_on="location_record_id", right_on="record_id")
        .with_columns(
            ctx.when(col("month").is_in([12, 1, 2]))
            .then(lit("Winter"))
            .otherwise(
                ctx.when(col("month").is_in([3, 4, 5]))
                .then(lit("Spring"))
                .otherwise(ctx.when(col("month").is_in([6, 7, 8])).then(lit("Summer")).otherwise(lit("Fall")))
            )
            .alias("season")
        )
        .group_by("region", "season")
        .agg(
            col("total_price").sum().alias("revenue"),
            col("order_id").n_unique().alias("orders"),
        )
        .sort(["region", "season"])
    )


def qc2_pandas_impl(ctx: DataFrameContext) -> Any:
    """QC2: Seasonal revenue comparison across regions."""
    import numpy as np

    params = get_parameters("QC2")
    start_year = params.get("start_year", 2023)
    end_year = params.get("end_year", 2024)

    ol = ctx.get_table("order_lines")
    dl = ctx.get_table("dim_locations")
    filtered = ol[(ol["order_date"].dt.year >= start_year) & (ol["order_date"].dt.year <= end_year)].copy()
    merged = filtered.merge(dl, left_on="location_record_id", right_on="record_id")
    month = merged["order_date"].dt.month
    conditions = [
        month.isin([12, 1, 2]),
        month.isin([3, 4, 5]),
        month.isin([6, 7, 8]),
    ]
    choices = ["Winter", "Spring", "Summer"]
    merged["season"] = np.select(conditions, choices, default="Fall")
    return (
        merged.groupby(["region", "season"], as_index=False)
        .agg(revenue=("total_price", "sum"), orders=("order_id", "nunique"))
        .sort_values(["region", "season"])
    )


# =============================================================================
# Query Registration
# =============================================================================


def _register_all_queries() -> None:
    """Register all 11 CoffeeShop DataFrame queries."""
    # Sales Analysis
    register_query(
        DataFrameQuery(
            query_id="SA1",
            query_name="Daily Revenue by Region",
            description="Daily revenue and order volume by region with NULLIF avg order value",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.AGGREGATE],
            expression_impl=sa1_expression_impl,
            pandas_impl=sa1_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="SA2",
            query_name="Top Products by Revenue",
            description="Top products by revenue for a given year with JOIN to dim_products",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.SORT],
            expression_impl=sa2_expression_impl,
            pandas_impl=sa2_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="SA3",
            query_name="Monthly Performance",
            description="Monthly orders, items sold, revenue with NULLIF derived metrics",
            categories=[
                QueryCategory.FILTER,
                QueryCategory.GROUP_BY,
                QueryCategory.AGGREGATE,
                QueryCategory.ANALYTICAL,
            ],
            expression_impl=sa3_expression_impl,
            pandas_impl=sa3_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="SA4",
            query_name="Revenue Share by Region",
            description="Revenue share by region using SUM OVER() window function",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.WINDOW],
            expression_impl=sa4_expression_impl,
            pandas_impl=sa4_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="SA5",
            query_name="Top Locations",
            description="Top-performing locations by revenue with multi-column GROUP BY",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.SORT],
            expression_impl=sa5_expression_impl,
            pandas_impl=sa5_pandas_impl,
        )
    )

    # Product Analysis
    register_query(
        DataFrameQuery(
            query_id="PR1",
            query_name="Product Mix",
            description="Product mix by subcategory with date validity check on dim_products",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.AGGREGATE],
            expression_impl=pr1_expression_impl,
            pandas_impl=pr1_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="PR2",
            query_name="Price Band Distribution",
            description="Price-band distribution using multi-way CASE WHEN binning",
            categories=[
                QueryCategory.FILTER,
                QueryCategory.GROUP_BY,
                QueryCategory.AGGREGATE,
                QueryCategory.ANALYTICAL,
            ],
            expression_impl=pr2_expression_impl,
            pandas_impl=pr2_pandas_impl,
        )
    )

    # Trend Analysis
    register_query(
        DataFrameQuery(
            query_id="TR1",
            query_name="Quarterly Trends",
            description="Quarterly revenue and order growth using FLOOR quarter calculation",
            categories=[
                QueryCategory.FILTER,
                QueryCategory.GROUP_BY,
                QueryCategory.AGGREGATE,
                QueryCategory.ANALYTICAL,
            ],
            expression_impl=tr1_expression_impl,
            pandas_impl=tr1_pandas_impl,
        )
    )

    # Time Analysis
    register_query(
        DataFrameQuery(
            query_id="TM1",
            query_name="Day-Part Cadence",
            description="Order cadence by day-part (Morning/Midday/Afternoon/Evening) for a region",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.ANALYTICAL],
            expression_impl=tm1_expression_impl,
            pandas_impl=tm1_pandas_impl,
        )
    )

    # Quality Checks
    register_query(
        DataFrameQuery(
            query_id="QC1",
            query_name="Lines per Order",
            description="Average lines and items per order with NULLIF protection",
            categories=[QueryCategory.FILTER, QueryCategory.AGGREGATE],
            expression_impl=qc1_expression_impl,
            pandas_impl=qc1_pandas_impl,
        )
    )
    register_query(
        DataFrameQuery(
            query_id="QC2",
            query_name="Seasonal Revenue",
            description="Seasonal revenue comparison across regions using CASE WHEN month mapping",
            categories=[QueryCategory.FILTER, QueryCategory.JOIN, QueryCategory.GROUP_BY, QueryCategory.ANALYTICAL],
            expression_impl=qc2_expression_impl,
            pandas_impl=qc2_pandas_impl,
        )
    )


_register_all_queries()



"""TPC benchmark metrics calculators.

This module provides standardized calculation of TPC-compliant benchmark metrics
including Power@Size, Throughput@Size, and QphH/QphDS composite metrics.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

import math
import statistics
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Sequence


class TPCMetricsCalculator:
    """Calculate TPC-compliant benchmark metrics.

    Implements metrics as defined in TPC-H and TPC-DS specifications:
    - Power@Size: Single-stream query throughput metric
    - Throughput@Size: Multi-stream concurrent throughput metric
    - QphH/QphDS: Composite metric combining power and throughput
    """

    @staticmethod
    def calculate_power_at_size(
        query_times_seconds: Sequence[float],
        scale_factor: float,
    ) -> float:
        """Calculate Power@Size metric (TPC-H/TPC-DS specification).

        Power@Size measures single-stream query performance:
        Power@Size = (SF * 3600) / geometric_mean(query_times)

        This represents queries per hour at the given scale factor.

        Args:
            query_times_seconds: List of query execution times in seconds
            scale_factor: Benchmark scale factor (e.g., 0.01, 1, 10, 100)

        Returns:
            Power@Size metric value, or 0.0 if calculation not possible
        """
        if not query_times_seconds:
            return 0.0

        # Filter out zero/negative times which would break geometric mean
        valid_times = [t for t in query_times_seconds if t > 0]
        if not valid_times:
            return 0.0

        geometric_mean = statistics.geometric_mean(valid_times)
        if geometric_mean <= 0:
            return 0.0

        return (scale_factor * 3600) / geometric_mean

    @staticmethod
    def calculate_throughput_at_size(
        total_queries: int,
        total_time_seconds: float,
        scale_factor: float,
        num_streams: int = 1,
    ) -> float:
        """Calculate Throughput@Size metric (TPC-H/TPC-DS specification).

        Throughput@Size measures concurrent multi-stream performance:
        Throughput@Size = (S * Q * SF * 3600) / T_s

        Where:
        - S = number of concurrent streams
        - Q = number of queries per stream
        - SF = scale factor
        - T_s = total elapsed time in seconds

        Args:
            total_queries: Total queries executed across all streams
            total_time_seconds: Total elapsed wall-clock time
            scale_factor: Benchmark scale factor
            num_streams: Number of concurrent streams

        Returns:
            Throughput@Size metric value, or 0.0 if calculation not possible
        """
        if total_time_seconds <= 0 or num_streams <= 0:
            return 0.0

        # Per TPC spec: (Streams * QueriesPerStream * SF * 3600) / TotalTime
        # But we have total_queries already = Streams * QueriesPerStream
        return (total_queries * scale_factor * 3600) / total_time_seconds

    @staticmethod
    def calculate_qph(power_at_size: float, throughput_at_size: float) -> float:
        """Calculate composite QphH/QphDS metric.

        The composite metric is the geometric mean of Power and Throughput:
        QphH = sqrt(Power@Size * Throughput@Size)

        Args:
            power_at_size: Power@Size metric value
            throughput_at_size: Throughput@Size metric value

        Returns:
            Composite QphH/QphDS metric, or 0.0 if either input is invalid
        """
        if power_at_size <= 0 or throughput_at_size <= 0:
            return 0.0

        return statistics.geometric_mean([power_at_size, throughput_at_size])

    @staticmethod
    def calculate_geometric_mean(times: Sequence[float]) -> float:
        """Calculate geometric mean of execution times.

        Args:
            times: Sequence of execution times (in any unit)

        Returns:
            Geometric mean of times, or 0.0 if not calculable
        """
        if not times:
            return 0.0

        valid_times = [t for t in times if t > 0]
        if not valid_times:
            return 0.0

        return statistics.geometric_mean(valid_times)


class TimingStatsCalculator:
    """Calculate timing statistics for query results."""

    @staticmethod
    def calculate(times_ms: Sequence[float]) -> dict[str, float]:
        """Calculate comprehensive timing statistics.

        Args:
            times_ms: List of execution times in milliseconds

        Returns:
            Dictionary containing:
            - total_ms: Sum of all times
            - avg_ms: Arithmetic mean
            - min_ms: Minimum time
            - max_ms: Maximum time
            - geometric_mean_ms: Geometric mean
            - stdev_ms: Standard deviation (0 if single value)
            - p50_ms: 50th percentile (median)
            - p90_ms: 90th percentile
            - p95_ms: 95th percentile
            - p99_ms: 99th percentile
        """
        if not times_ms:
            return {}

        times_list = list(times_ms)
        sorted_times = sorted(times_list)
        n = len(sorted_times)

        def percentile(p: float) -> float:
            """Calculate percentile using nearest-rank method."""
            if n == 0:
                return 0.0
            k = max(1, math.ceil(n * p))
            k = min(k, n)  # Clamp to valid rank
            k -= 1  # Convert rank to 0-based index
            return sorted_times[k]

        # Calculate geometric mean (only for positive values)
        positive_times = [t for t in times_list if t > 0]
        geom_mean = statistics.geometric_mean(positive_times) if positive_times else 0.0

        return {
            "total_ms": sum(times_list),
            "avg_ms": statistics.mean(times_list),
            "min_ms": min(times_list),
            "max_ms": max(times_list),
            "geometric_mean_ms": geom_mean,
            "stdev_ms": statistics.stdev(times_list) if n > 1 else 0.0,
            "p50_ms": percentile(0.50),
            "p90_ms": percentile(0.90),
            "p95_ms": percentile(0.95),
            "p99_ms": percentile(0.99),
        }

    @staticmethod
    def calculate_seconds(times_seconds: Sequence[float]) -> dict[str, float]:
        """Calculate timing statistics with times in seconds.

        Convenience method that returns results in seconds instead of milliseconds.

        Args:
            times_seconds: List of execution times in seconds

        Returns:
            Same structure as calculate() but with _s suffix instead of _ms
        """
        if not times_seconds:
            return {}

        times_ms = [t * 1000 for t in times_seconds]
        stats_ms = TimingStatsCalculator.calculate(times_ms)

        # Convert back to seconds
        return {key.replace("_ms", "_s"): value / 1000 for key, value in stats_ms.items()}



"""Setup and connection routines for InfluxDB.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

import logging
from typing import Any

from .client import InfluxDBConnection

logger = logging.getLogger(__name__)


class InfluxDBSetupMixin:
    """Provide setup and connection helpers for InfluxDB."""

    def _setup_connection_params(self, config: dict[str, Any]) -> None:
        """Setup connection parameters from config.

        Extracts and validates connection parameters including host, port, token, org,
        database, SSL setting, and deployment mode. Token is required for authentication;
        if not provided in config, the adapter will attempt to use INFLUXDB_TOKEN
        environment variable during connection.

        Args:
            config: Configuration dictionary with connection parameters:
                - host: InfluxDB server hostname (default: localhost)
                - port: Server port (default: 8086)
                - token: Authentication token (required; can be set via INFLUXDB_TOKEN env var)
                - org: Organization name (optional, required for some deployments)
                - database: Database/bucket name (default: benchbox)
                - ssl: Use SSL/TLS connection (default: True)
                - mode: Deployment mode 'core' or 'cloud' (default: cloud)
        """
        # Connection parameters
        self.host = config.get("host", "localhost")
        self.port = config.get("port", 8086)
        self.token = config.get("token")
        self.org = config.get("org")
        self.database = config.get("database", "benchbox")
        self.ssl = config.get("ssl", True)
        self.mode = config.get("mode", "cloud")

        # Validate token is provided
        if not self.token:
            logger.warning("No InfluxDB token provided. Set --token or INFLUXDB_TOKEN environment variable.")

    def create_connection(self, **connection_config) -> InfluxDBConnection:
        """Create InfluxDB connection.

        Args:
            **connection_config: Override connection parameters

        Returns:
            Connected InfluxDBConnection instance

        Raises:
            ConnectionError: If connection fails
        """
        self.log_operation_start("InfluxDB connection", f"mode: {self.mode}")

        # Get connection parameters with overrides
        host = connection_config.get("host", self.host)
        port = connection_config.get("port", self.port)
        token = connection_config.get("token", self.token)
        database = connection_config.get("database", self.database)
        ssl = connection_config.get("ssl", self.ssl)
        org = connection_config.get("org", self.org)

        try:
            connection = InfluxDBConnection(
                host=host,
                token=token,
                database=database,
                port=port,
                ssl=ssl,
                org=org,
            )
            connection.connect()

            # Test connection
            if connection.test_connection():
                self.logger.info(f"Connected to InfluxDB at {host}:{port}")
            else:
                raise ConnectionError("Connection test failed")

            return connection

        except Exception as e:
            self.logger.error(f"Failed to connect to InfluxDB: {e}")
            raise

    def close_connection(self, connection: Any) -> None:
        """Close InfluxDB connection.

        Args:
            connection: InfluxDBConnection to close
        """
        try:
            if connection and hasattr(connection, "close"):
                connection.close()
        except Exception as e:
            self.logger.warning(f"Error closing connection: {e}")

    def handle_existing_database(self, **connection_config) -> None:
        """Handle existing database based on force_recreate setting.

        InfluxDB 3.x manages databases (buckets) differently:
        - Core/OSS: Databases are created automatically
        - Cloud: Databases are managed via the InfluxDB UI or API

        For benchmarking, we typically work with existing databases.

        Args:
            **connection_config: Connection configuration
        """
        if getattr(self, "force_recreate", False):
            self.logger.warning(
                "InfluxDB does not support database recreation via SQL. "
                "Database must be managed via InfluxDB UI or API."
            )

    def get_database_path(self, **connection_config) -> str | None:
        """Get database path for local mode persistence.

        InfluxDB doesn't use file-based databases like DuckDB/SQLite.
        This method is provided for interface compatibility.

        Returns:
            None (InfluxDB doesn't use file paths)
        """
        return None


__all__ = ["InfluxDBSetupMixin"]



"""Transaction Primitives data generator.

Generates staging data and bulk load files for transaction primitives benchmark.
Reuses TPC-H base data to avoid duplication.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

from pathlib import Path

from benchbox.core.primitives import generator_base as _generator_base
from benchbox.core.primitives.generator_base import PrimitivesDataGeneratorBase

# Backward compatibility: tests and downstream code monkeypatch these symbols
# on this module directly.
DataGenerationManifest = _generator_base.DataGenerationManifest
resolve_compression_metadata = _generator_base.resolve_compression_metadata


class TransactionPrimitivesDataGenerator(PrimitivesDataGeneratorBase):
    """Transaction Primitives data generator.

    Reuses TPC-H data for base tables and generates staging tables
    and bulk load files for write operations testing.
    """

    _benchmark_name = "transaction_primitives"
    _display_name = "Transaction Primitives"
    _auxiliary_dir = "transaction_primitives_auxiliary"
    _logger_name = "benchbox.core.transaction_primitives.generator"

    def _write_manifest(self, table_paths: dict[str, Path]) -> None:
        """Write manifest while honoring module-level monkeypatch hooks."""
        _generator_base.DataGenerationManifest = DataGenerationManifest
        _generator_base.resolve_compression_metadata = resolve_compression_metadata
        super()._write_manifest(table_paths)


__all__ = [
    "TransactionPrimitivesDataGenerator",
    "DataGenerationManifest",
    "resolve_compression_metadata",
]



"""Dimension table generation mixin for TPC-DI."""

from __future__ import annotations

import csv
import random
from datetime import datetime, time, timedelta

from ..financial_data import generate_realistic_tax_rates


class DimensionGenerationMixin:
    """Generate dimension table datasets for the TPC-DI benchmark."""

    def _generate_dimdate_data(self) -> str:
        """Generate the DimDate dimension data with buffered I/O."""
        file_path = self.output_dir / "DimDate.tbl"

        start_date = datetime(2010, 1, 1)
        end_date = datetime(2025, 12, 31)
        total_days = (end_date - start_date).days + 1

        with open(file_path, "w", newline="", buffering=self.buffer_size) as f:
            writer = csv.writer(f, delimiter="|")

            current_date = start_date
            sk_date_id = 1
            chunk_buffer = []

            while current_date <= end_date:
                date_value = current_date.date()
                date_desc = current_date.strftime("%Y-%m-%d")

                calendar_year_id = current_date.year
                calendar_year_desc = str(current_date.year)
                calendar_qtr_id = (current_date.month - 1) // 3 + 1
                calendar_qtr_desc = f"Q{calendar_qtr_id} {current_date.year}"
                calendar_month_id = current_date.month
                calendar_month_desc = current_date.strftime("%B %Y")
                calendar_week_id = current_date.isocalendar()[1]
                calendar_week_desc = f"Week {calendar_week_id} {current_date.year}"
                day_of_week_num = current_date.weekday() + 1
                day_of_week_desc = current_date.strftime("%A")

                # Simplified fiscal year (same as calendar)
                fiscal_year_id = calendar_year_id
                fiscal_year_desc = calendar_year_desc
                fiscal_qtr_id = calendar_qtr_id
                fiscal_qtr_desc = calendar_qtr_desc

                # Simple holiday flag (just major US holidays)
                holiday_flag = (
                    (current_date.month == 1 and current_date.day == 1)  # Year
                    or (current_date.month == 7 and current_date.day == 4)  # Independence Day
                    or (current_date.month == 12 and current_date.day == 25)  # Christmas
                )

                row = [
                    sk_date_id,
                    date_value,
                    date_desc,
                    calendar_year_id,
                    calendar_year_desc,
                    calendar_qtr_id,
                    calendar_qtr_desc,
                    calendar_month_id,
                    calendar_month_desc,
                    calendar_week_id,
                    calendar_week_desc,
                    day_of_week_num,
                    day_of_week_desc,
                    fiscal_year_id,
                    fiscal_year_desc,
                    fiscal_qtr_id,
                    fiscal_qtr_desc,
                    holiday_flag,
                ]

                chunk_buffer.append(row)

                # Write in chunks for better I/O performance
                if len(chunk_buffer) >= self.chunk_size:
                    writer.writerows(chunk_buffer)
                    chunk_buffer.clear()
                    # Simple progress logging every 1000 records
                    if self.enable_progress and sk_date_id % 1000 == 0:
                        self.logger.info(f"DimDate: generated {sk_date_id:,} of {total_days:,} records")

                current_date += timedelta(days=1)
                sk_date_id += 1

            # Write remaining rows
            if chunk_buffer:
                writer.writerows(chunk_buffer)

        self.generation_stats["records_generated"] += total_days
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimtime_data(self) -> str:
        """Generate the DimTime dimension data."""
        file_path = self.output_dir / "DimTime.tbl"

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            sk_time_id = 1

            for hour in range(24):
                for minute in range(0, 60, 5):  # Every 5 minutes
                    time_value = time(hour, minute)
                    hour_desc = f"Hour {hour:02d}"
                    minute_desc = f"Minute {minute:02d}"
                    second_desc = "Second 00"

                    # Market hours: 9:30 AM to 4:00 PM ET
                    market_hours_flag = (9 <= hour < 16) or (hour == 9 and minute >= 30)

                    # Office hours: 8:00 AM to 6:00 PM
                    office_hours_flag = 8 <= hour < 18

                    row = [
                        sk_time_id,
                        time_value,
                        hour,
                        hour_desc,
                        minute,
                        minute_desc,
                        0,
                        second_desc,
                        market_hours_flag,
                        office_hours_flag,
                    ]

                    writer.writerow(row)
                    sk_time_id += 1

        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimcompany_data(self) -> str:
        """Generate the DimCompany dimension data with chunked processing."""
        file_path = self.output_dir / "DimCompany.tbl"
        num_companies = int(self.base_companies * self.scale_factor)

        with open(file_path, "w", newline="", buffering=self.buffer_size) as f:
            writer = csv.writer(f, delimiter="|")

            # Process in chunks to manage memory
            for chunk_start in range(1, num_companies + 1, self.chunk_size):
                chunk_end = min(chunk_start + self.chunk_size, num_companies + 1)
                chunk_rows = []

                for i in range(chunk_start, chunk_end):
                    sk_company_id = i
                    company_id = i
                    status = random.choice(self._statuses)
                    name = f"Company {i:04d} Inc."
                    industry = random.choice(self._industries)
                    sp_rating = random.choice(self._sp_ratings)
                    is_low_grade = sp_rating in ["BB+", "BB", "BB-"]
                    # Generate market cap based on company size and rating
                    # Higher rated companies tend to have larger market caps
                    base_market_cap = random.uniform(100_000_000, 50_000_000_000)  # $100M to $50B
                    if not is_low_grade:
                        market_cap = round(base_market_cap * random.uniform(1.2, 2.0), 2)
                    else:
                        market_cap = round(base_market_cap * random.uniform(0.5, 1.0), 2)
                    ceo = f"CEO {i:04d}"

                    address_line1 = f"{random.randint(1, 9999)} Main Street"
                    address_line2 = ""
                    postal_code = f"{random.randint(10000, 99999)}"
                    city = f"City{i % 100}"
                    state_prov = random.choice(self._us_states)
                    country = "USA"
                    description = f"Description for {name}"
                    founding_date = datetime(
                        random.randint(1950, 2020),
                        random.randint(1, 12),
                        random.randint(1, 28),
                    ).date()

                    is_current = True
                    batch_id = 1
                    effective_date = datetime(2010, 1, 1).date()
                    end_date = datetime(9999, 12, 31).date()

                    row = [
                        sk_company_id,
                        company_id,
                        status,
                        name,
                        industry,
                        sp_rating,
                        is_low_grade,
                        market_cap,
                        ceo,
                        address_line1,
                        address_line2,
                        postal_code,
                        city,
                        state_prov,
                        country,
                        description,
                        founding_date,
                        is_current,
                        batch_id,
                        effective_date,
                        end_date,
                    ]

                    chunk_rows.append(row)

                writer.writerows(chunk_rows)
                # Simple progress logging
                if self.enable_progress:
                    self.logger.info(f"DimCompany: generated {chunk_end - 1:,} of {num_companies:,} records")
                self.generation_stats["chunks_processed"] += 1

                # Memory management
                if self._check_memory_usage():
                    self._cleanup_memory()

        self.generation_stats["records_generated"] += num_companies
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimsecurity_data(self) -> str:
        """Generate the DimSecurity dimension data."""
        file_path = self.output_dir / "DimSecurity.tbl"
        num_securities = int(self.base_securities * self.scale_factor)
        num_companies = int(self.base_companies * self.scale_factor)

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            for i in range(1, num_securities + 1):
                sk_security_id = i
                symbol = f"SYM{i:04d}"
                issue = "S"  # Stock
                status = random.choice(self._statuses)
                name = f"Security {i:04d}"
                exchange_id = random.choice(["NYSE", "NASDAQ", "AMEX"])
                sk_company_id = random.randint(1, num_companies)
                shares_outstanding = random.randint(1000000, 1000000000)
                first_trade = datetime(
                    random.randint(2000, 2020),
                    random.randint(1, 12),
                    random.randint(1, 28),
                ).date()
                first_trade_on_exchange = first_trade
                dividend = round(random.uniform(0, 5.0), 2)

                is_current = True
                batch_id = 1
                effective_date = datetime(2010, 1, 1).date()
                end_date = datetime(9999, 12, 31).date()

                row = [
                    sk_security_id,
                    symbol,
                    issue,
                    status,
                    name,
                    exchange_id,
                    sk_company_id,
                    shares_outstanding,
                    first_trade,
                    first_trade_on_exchange,
                    dividend,
                    is_current,
                    batch_id,
                    effective_date,
                    end_date,
                ]

                writer.writerow(row)

        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimcustomer_data(self) -> str:
        """Generate the DimCustomer dimension data."""
        file_path = self.output_dir / "DimCustomer.tbl"
        num_customers = int(self.base_customers * self.scale_factor)

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            for i in range(1, num_customers + 1):
                sk_customer_id = i
                customer_id = i
                tax_id = f"{random.randint(100000000, 999999999)}"
                status = random.choice(self._statuses)
                last_name = f"LastName{i:05d}"
                first_name = f"FirstName{i:05d}"
                middle_initial = random.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
                gender = random.choice(["M", "F"])
                tier = random.choices([1, 2, 3], weights=[0.6, 0.3, 0.1])[0]
                dob = datetime(
                    random.randint(1930, 2000),
                    random.randint(1, 12),
                    random.randint(1, 28),
                ).date()

                # Address information
                address_line1 = f"{random.randint(1, 9999)} Customer Street"
                address_line2 = ""
                postal_code = f"{random.randint(10000, 99999)}"
                city = f"City{i % 500}"
                state_prov = random.choice(self._us_states)
                country = "USA"

                # Contact information
                phone1 = f"{random.randint(100, 999)}-{random.randint(100, 999)}-{random.randint(1000, 9999)}"
                phone2 = ""
                phone3 = ""
                email1 = f"customer{i}@email.com"
                email2 = ""

                # Tax information
                national_tax_rate_desc = "Federal Tax"
                national_tax_rate = round(random.uniform(0.15, 0.35), 5)
                local_tax_rate_desc = "State Tax"
                local_tax_rate = round(random.uniform(0.0, 0.10), 5)

                agency_id = "AGY001"
                credit_rating = random.randint(300, 850)
                net_worth = random.randint(10000, 10000000)
                marketing_nameplate = f"Customer {i}"

                is_current = True
                batch_id = 1
                effective_date = datetime(2010, 1, 1).date()
                end_date = datetime(9999, 12, 31).date()

                row = [
                    sk_customer_id,
                    customer_id,
                    tax_id,
                    status,
                    last_name,
                    first_name,
                    middle_initial,
                    gender,
                    tier,
                    dob,
                    address_line1,
                    address_line2,
                    postal_code,
                    city,
                    state_prov,
                    country,
                    phone1,
                    phone2,
                    phone3,
                    email1,
                    email2,
                    national_tax_rate_desc,
                    national_tax_rate,
                    local_tax_rate_desc,
                    local_tax_rate,
                    agency_id,
                    credit_rating,
                    net_worth,
                    marketing_nameplate,
                    is_current,
                    batch_id,
                    effective_date,
                    end_date,
                ]

                writer.writerow(row)

        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimaccount_data(self) -> str:
        """Generate the DimAccount dimension data."""
        file_path = self.output_dir / "DimAccount.tbl"
        num_accounts = int(self.base_accounts * self.scale_factor)
        num_customers = int(self.base_customers * self.scale_factor)

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            for i in range(1, num_accounts + 1):
                sk_account_id = i
                account_id = i
                sk_broker_id = random.randint(1, 100)  # Assume 100 brokers
                sk_customer_id = random.randint(1, num_customers)
                status = random.choice(self._statuses)
                account_desc = f"Account {i:06d}"
                tax_status = random.randint(0, 2)  # 0=Taxable, 1=Tax Deferred, 2=Tax Free

                is_current = True
                batch_id = 1
                effective_date = datetime(2010, 1, 1).date()
                end_date = datetime(9999, 12, 31).date()

                row = [
                    sk_account_id,
                    account_id,
                    sk_broker_id,
                    sk_customer_id,
                    status,
                    account_desc,
                    tax_status,
                    is_current,
                    batch_id,
                    effective_date,
                    end_date,
                ]

                writer.writerow(row)

        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_industry_data(self) -> str:
        """Generate Industry reference data."""
        file_path = self.output_dir / "Industry.tbl"

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            industry_data = self.financial_patterns.get_industry_data()
            for industry_id, industry_name, sector_code in industry_data:
                row = [industry_id, industry_name, sector_code]
                writer.writerow(row)

        self.generation_stats["records_generated"] += len(industry_data)
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_statustype_data(self) -> str:
        """Generate StatusType reference data."""
        file_path = self.output_dir / "StatusType.tbl"

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            status_data = self.financial_patterns.get_status_types()
            for status_id, status_name in status_data:
                row = [status_id, status_name]
                writer.writerow(row)

        self.generation_stats["records_generated"] += len(status_data)
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_taxrate_data(self) -> str:
        """Generate TaxRate reference data."""
        file_path = self.output_dir / "TaxRate.tbl"

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            tax_data = generate_realistic_tax_rates()
            for tax_id, tax_name, tax_rate in tax_data:
                row = [tax_id, tax_name, tax_rate]
                writer.writerow(row)

        self.generation_stats["records_generated"] += len(tax_data)
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_tradetype_data(self) -> str:
        """Generate TradeType reference data."""
        file_path = self.output_dir / "TradeType.tbl"

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            trade_data = self.financial_patterns.get_trade_types()
            for type_id, type_name, is_sell, is_market in trade_data:
                row = [type_id, type_name, is_sell, is_market]
                writer.writerow(row)

        self.generation_stats["records_generated"] += len(trade_data)
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)

    def _generate_dimbroker_data(self) -> str:
        """Generate DimBroker dimension data with realistic hierarchy."""
        file_path = self.output_dir / "DimBroker.tbl"
        num_brokers = max(100, int(self.scale_factor * 50))  # Minimum 100 brokers

        with open(file_path, "w", newline="") as f:
            writer = csv.writer(f, delimiter="|")

            # Generate manager hierarchy
            managers = []
            for i in range(1, min(21, num_brokers // 5) + 1):  # Up to 20 managers
                managers.append(i)

            for i in range(1, num_brokers + 1):
                sk_broker_id = i
                broker_id = i

                # Assign manager (some brokers are managers themselves)
                if i in managers:
                    manager_id = None  # Top-level managers
                else:
                    manager_id = random.choice(managers)

                first_name = random.choice(self.financial_patterns.first_names)
                last_name = random.choice(self.financial_patterns.last_names)
                middle_initial = random.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ")

                # Generate branch and office
                branch = f"Branch {((i - 1) // 20) + 1:02d}"  # 20 brokers per branch
                office = f"Office {random.choice(['A', 'B', 'C', 'D'])}"
                phone = f"{random.randint(100, 999)}-{random.randint(100, 999)}-{random.randint(1000, 9999)}"

                is_current = True
                batch_id = 1
                effective_date = datetime(2010, 1, 1).date()
                end_date = datetime(9999, 12, 31).date()

                row = [
                    sk_broker_id,
                    broker_id,
                    manager_id,
                    first_name,
                    last_name,
                    middle_initial,
                    branch,
                    office,
                    phone,
                    is_current,
                    batch_id,
                    effective_date,
                    end_date,
                ]
                writer.writerow(row)

        self.generation_stats["records_generated"] += num_brokers
        file_path = self.compress_existing_file(file_path, remove_original=True)
        return str(file_path)


__all__ = ["DimensionGenerationMixin"]



"""Amazon Athena for Apache Spark platform adapter.

Athena for Apache Spark is AWS's interactive Spark service with sub-second
startup times. Unlike EMR Serverless or Glue, it uses a notebook-style
execution model with persistent sessions.

Key Features:
- Sub-second startup: Pre-provisioned Spark capacity
- Interactive: Notebook-style session execution
- Serverless: No cluster management required
- S3 integration: Native S3 and Glue Data Catalog support
- Cost-effective: Pay per DPU-hour during session

Usage:
    from benchbox.platforms.aws import AthenaSparkAdapter

    adapter = AthenaSparkAdapter(
        workgroup="spark-workgroup",
        s3_staging_dir="s3://my-bucket/benchbox-data",
    )

    # Run TPC-H benchmark
    adapter.create_schema("tpch_sf1")
    adapter.load_data(["lineitem", "orders", ...], source_dir)
    result = adapter.execute_query("SELECT * FROM lineitem LIMIT 10")

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

import json
import logging
import time
from pathlib import Path
from typing import TYPE_CHECKING, Any

from benchbox.utils.clock import elapsed_seconds, mono_time

if TYPE_CHECKING:
    from benchbox.core.tuning.interface import (
        UnifiedTuningConfiguration,
    )

from benchbox.core.exceptions import ConfigurationError
from benchbox.platforms.base import DriverIsolationCapability, PlatformAdapter
from benchbox.platforms.base.cloud_spark import (
    CloudSparkConfigMixin,
    CloudSparkStaging,
    SparkTuningMixin,
)
from benchbox.platforms.base.cloud_spark.config import CloudPlatform
from benchbox.utils.dependencies import (
    check_platform_dependencies,
    get_dependency_error_message,
)

try:
    import boto3
    from botocore.exceptions import ClientError

    BOTO3_AVAILABLE = True
except ImportError:
    boto3 = None
    ClientError = Exception
    BOTO3_AVAILABLE = False

logger = logging.getLogger(__name__)


class AthenaSparkSessionState:
    """Athena Spark session state constants.

    Reference: https://docs.aws.amazon.com/athena/latest/APIReference/API_SessionStatus.html
    """

    CREATING = "CREATING"
    CREATED = "CREATED"
    IDLE = "IDLE"
    BUSY = "BUSY"
    TERMINATING = "TERMINATING"
    TERMINATED = "TERMINATED"
    DEGRADED = "DEGRADED"
    FAILED = "FAILED"

    # States where session is usable
    READY_STATES = {IDLE, CREATED}

    # Terminal states
    TERMINAL_STATES = {TERMINATED, FAILED}


class AthenaSparkCalculationState:
    """Athena Spark calculation (query) state constants.

    Reference: https://docs.aws.amazon.com/athena/latest/APIReference/API_CalculationStatus.html
    """

    CREATING = "CREATING"
    CREATED = "CREATED"
    QUEUED = "QUEUED"
    RUNNING = "RUNNING"
    CANCELING = "CANCELING"
    CANCELED = "CANCELED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"

    # Terminal states
    TERMINAL_STATES = {COMPLETED, FAILED, CANCELED}

    # Success states
    SUCCESS_STATES = {COMPLETED}


class AthenaSparkAdapter(CloudSparkConfigMixin, SparkTuningMixin, PlatformAdapter):
    """Amazon Athena for Apache Spark platform adapter.

    Athena Spark provides interactive Spark execution with sub-second startup.
    It uses a session-based model where you start a session, run calculations
    (queries), and terminate the session when done.

    Execution Model:
    - Start a session in a Spark-enabled workgroup
    - Submit calculations (SQL or PySpark code)
    - Results are written to S3 automatically
    - Terminate session when complete

    Key Features:
    - Sub-second startup with pre-provisioned capacity
    - Interactive session-based execution
    - Native Glue Data Catalog integration
    - S3 integration for data and results

    Billing:
    - DPU-hour: ~$0.35/hour per DPU
    - Minimum: 1 DPU
    - Billed per session duration
    """

    driver_isolation_capability = DriverIsolationCapability.NOT_FEASIBLE

    # CloudSparkConfigMixin: Uses EMR config as base for Athena Spark
    cloud_platform = CloudPlatform.EMR

    def __init__(
        self,
        workgroup: str | None = None,
        s3_staging_dir: str | None = None,
        region: str = "us-east-1",
        database: str | None = None,
        engine_version: str = "PySpark engine version 3",
        session_idle_timeout_minutes: int = 15,
        coordinator_dpu_size: int = 1,
        max_concurrent_dpus: int = 20,
        default_executor_dpu_size: int = 1,
        timeout_minutes: int = 60,
        notebook_version: str | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize the Athena Spark adapter.

        Args:
            workgroup: Athena workgroup name (must be Spark-enabled).
            s3_staging_dir: S3 path for data staging (required, e.g., s3://bucket/path).
            region: AWS region (default: us-east-1).
            database: Glue Data Catalog database name (default: benchbox).
            engine_version: Spark engine version (default: PySpark engine version 3).
            session_idle_timeout_minutes: Session idle timeout (default: 15).
            coordinator_dpu_size: Coordinator DPU size (default: 1).
            max_concurrent_dpus: Maximum concurrent DPUs (default: 20).
            default_executor_dpu_size: Default executor DPU size (default: 1).
            timeout_minutes: Calculation timeout in minutes (default: 60).
            notebook_version: Jupyter notebook version (optional).
            **kwargs: Additional platform options.
        """
        if not BOTO3_AVAILABLE:
            deps_satisfied, missing = check_platform_dependencies("athena-spark")
            if not deps_satisfied:
                raise ConfigurationError(get_dependency_error_message("athena-spark", missing))

        if not workgroup:
            raise ConfigurationError("workgroup is required for Athena Spark. Must be a Spark-enabled workgroup.")

        if not s3_staging_dir:
            raise ConfigurationError("s3_staging_dir is required (e.g., s3://bucket/path)")

        if not s3_staging_dir.startswith("s3://"):
            raise ConfigurationError(f"Invalid S3 path: {s3_staging_dir}. Must start with s3://")

        # Parse S3 path
        s3_parts = s3_staging_dir[5:].split("/", 1)
        self.s3_bucket = s3_parts[0]
        self.s3_prefix = s3_parts[1] if len(s3_parts) > 1 else ""

        self.workgroup = workgroup
        self.s3_staging_dir = s3_staging_dir.rstrip("/")
        self.region = region
        self.database = database or "benchbox"
        self.engine_version = engine_version
        self.session_idle_timeout_minutes = session_idle_timeout_minutes
        self.coordinator_dpu_size = coordinator_dpu_size
        self.max_concurrent_dpus = max_concurrent_dpus
        self.default_executor_dpu_size = default_executor_dpu_size
        self.timeout_minutes = timeout_minutes
        self.notebook_version = notebook_version

        # Initialize staging using cloud-spark shared infrastructure
        self._staging: CloudSparkStaging | None = None
        try:
            self._staging = CloudSparkStaging.from_uri(self.s3_staging_dir)
        except Exception as e:
            logger.warning(f"Failed to initialize S3 staging: {e}")

        # Clients (lazy initialization)
        self._athena_client: Any = None
        self._glue_client: Any = None
        self._s3_client: Any = None

        # Session management
        self._session_id: str | None = None

        # Metrics tracking
        self._query_count = 0
        self._total_execution_time_seconds = 0.0
        self._total_dpu_hours = 0.0

        # Benchmark configuration (set via configure_for_benchmark)
        self._benchmark_type: str | None = None
        self._scale_factor: float = 1.0
        self._spark_config: dict[str, str] = {}

        super().__init__(**kwargs)

    def _get_athena_client(self) -> Any:
        """Get or create Athena client."""
        if self._athena_client is None:
            self._athena_client = boto3.client("athena", region_name=self.region)
        return self._athena_client

    def _get_glue_client(self) -> Any:
        """Get or create Glue client."""
        if self._glue_client is None:
            self._glue_client = boto3.client("glue", region_name=self.region)
        return self._glue_client

    def _get_s3_client(self) -> Any:
        """Get or create S3 client."""
        if self._s3_client is None:
            self._s3_client = boto3.client("s3", region_name=self.region)
        return self._s3_client

    def get_platform_info(self, connection: Any = None) -> dict[str, Any]:
        """Return platform metadata.

        Args:
            connection: Not used (Athena Spark manages sessions internally).

        Returns:
            Dict with platform information including name, version, and capabilities.
        """
        return {
            "platform": "athena-spark",
            "display_name": "Amazon Athena for Apache Spark",
            "vendor": "AWS",
            "type": "interactive_spark",
            "region": self.region,
            "workgroup": self.workgroup,
            "engine_version": self.engine_version,
            "engine_version_source": "config",
            "supports_sql": True,
            "supports_dataframe": True,
            "billing_model": "DPU-hour",
            "session_id": self._session_id,
        }

    def create_connection(self, **kwargs: Any) -> Any:
        """Start an Athena Spark session.

        Returns:
            Dict with session status and ID.

        Raises:
            ConfigurationError: If session creation fails.
        """
        client = self._get_athena_client()

        try:
            # Check if we already have an active session
            if self._session_id:
                session_status = self._get_session_status()
                if session_status in AthenaSparkSessionState.READY_STATES:
                    logger.info(f"Using existing session: {self._session_id}")
                    return {
                        "status": "connected",
                        "session_id": self._session_id,
                        "session_state": session_status,
                    }

            # Start a new session
            logger.info(f"Starting Athena Spark session in workgroup: {self.workgroup}")

            session_config = {
                "CoordinatorDpuSize": self.coordinator_dpu_size,
                "MaxConcurrentDpus": self.max_concurrent_dpus,
                "DefaultExecutorDpuSize": self.default_executor_dpu_size,
            }

            if self.notebook_version:
                session_config["NotebookVersion"] = self.notebook_version

            response = client.start_session(
                WorkGroup=self.workgroup,
                EngineConfiguration=session_config,
                SessionIdleTimeoutInMinutes=self.session_idle_timeout_minutes,
            )

            self._session_id = response["SessionId"]
            session_state = response["State"]

            # Wait for session to become ready
            self._wait_for_session_ready()

            logger.info(f"Athena Spark session started: {self._session_id}")
            return {
                "status": "connected",
                "session_id": self._session_id,
                "session_state": session_state,
                "workgroup": self.workgroup,
            }

        except ClientError as e:
            error_code = e.response.get("Error", {}).get("Code", "Unknown")
            error_message = e.response.get("Error", {}).get("Message", str(e))

            if error_code == "InvalidRequestException":
                raise ConfigurationError(
                    f"Invalid Athena Spark configuration: {error_message}. "
                    f"Ensure workgroup '{self.workgroup}' is Spark-enabled."
                ) from e
            raise ConfigurationError(f"Failed to start Athena Spark session: {error_message}") from e

    def _get_session_status(self) -> str:
        """Get current session status."""
        if not self._session_id:
            return AthenaSparkSessionState.TERMINATED

        client = self._get_athena_client()

        try:
            response = client.get_session_status(SessionId=self._session_id)
            return response["Status"]["State"]
        except Exception:
            return AthenaSparkSessionState.TERMINATED

    def _wait_for_session_ready(self, timeout_seconds: int = 300) -> None:
        """Wait for session to become ready.

        Args:
            timeout_seconds: Maximum time to wait.

        Raises:
            ConfigurationError: If session fails or times out.
        """
        client = self._get_athena_client()
        start_time = mono_time()

        while elapsed_seconds(start_time) < timeout_seconds:
            response = client.get_session_status(SessionId=self._session_id)
            state = response["Status"]["State"]

            if state in AthenaSparkSessionState.READY_STATES:
                return

            if state in AthenaSparkSessionState.TERMINAL_STATES:
                reason = response["Status"].get("StateChangeReason", "Unknown")
                raise ConfigurationError(f"Session failed: {state} - {reason}")

            time.sleep(2)

        raise ConfigurationError(f"Session startup timed out after {timeout_seconds}s")

    def create_schema(self, schema_name: str | None = None) -> None:
        """Create Glue database if it doesn't exist.

        Args:
            schema_name: Database name (uses self.database if not provided).
        """
        database = schema_name or self.database
        glue_client = self._get_glue_client()

        try:
            glue_client.get_database(Name=database)
            logger.info(f"Database '{database}' already exists")
        except glue_client.exceptions.EntityNotFoundException:
            # Create database
            location_uri = f"{self.s3_staging_dir}/databases/{database}/"
            glue_client.create_database(
                DatabaseInput={
                    "Name": database,
                    "Description": "BenchBox benchmark database",
                    "LocationUri": location_uri,
                }
            )
            logger.info(f"Created database '{database}'")

    def _submit_calculation(
        self,
        code: str,
        code_type: str = "SQL",
        wait_for_completion: bool = True,
    ) -> tuple[str, str]:
        """Submit a calculation to the Athena Spark session.

        Args:
            code: SQL or Python code to execute.
            code_type: Type of code ("SQL" or "PYTHON").
            wait_for_completion: Whether to wait for completion.

        Returns:
            Tuple of (calculation_id, final_state).
        """
        if not self._session_id:
            raise ConfigurationError("No active session. Call create_connection() first.")

        client = self._get_athena_client()

        # For SQL, wrap in spark.sql() for proper execution
        if code_type == "SQL":
            # Ensure we're using the correct database
            execution_code = f"""
spark.sql("USE {self.database}")
result = spark.sql('''{code}''')
result.show(100, truncate=False)
"""
        else:
            execution_code = code

        response = client.start_calculation_execution(
            SessionId=self._session_id,
            CodeBlock=execution_code,
        )

        calculation_id = response["CalculationExecutionId"]
        state = response["State"]

        if wait_for_completion:
            state = self._wait_for_calculation_complete(calculation_id)

        return calculation_id, state

    def _wait_for_calculation_complete(
        self,
        calculation_id: str,
        timeout_seconds: int | None = None,
    ) -> str:
        """Wait for calculation to complete.

        Args:
            calculation_id: The calculation execution ID.
            timeout_seconds: Maximum time to wait.

        Returns:
            Final calculation state.
        """
        client = self._get_athena_client()
        timeout = timeout_seconds or self.timeout_minutes * 60
        start_time = mono_time()

        while elapsed_seconds(start_time) < timeout:
            response = client.get_calculation_execution_status(CalculationExecutionId=calculation_id)
            state = response["Status"]["State"]

            if state in AthenaSparkCalculationState.TERMINAL_STATES:
                return state

            time.sleep(2)

        raise RuntimeError(f"Calculation timed out after {timeout}s")

    def _get_calculation_result(self, calculation_id: str) -> list[dict[str, Any]]:
        """Get calculation results.

        Args:
            calculation_id: The calculation execution ID.

        Returns:
            List of result rows as dicts.
        """
        client = self._get_athena_client()

        try:
            response = client.get_calculation_execution(CalculationExecutionId=calculation_id)

            # Results are in the Result field
            result = response.get("Result", {})
            result_s3_uri = result.get("ResultS3Uri")

            if result_s3_uri:
                # Fetch results from S3
                return self._fetch_results_from_s3(result_s3_uri)

            # Try to get stdout if no S3 results
            stdout = result.get("StdOutS3Uri")
            if stdout:
                return self._fetch_results_from_s3(stdout)

            return []

        except Exception as e:
            logger.warning(f"Could not get calculation results: {e}")
            return []

    def _fetch_results_from_s3(self, s3_uri: str) -> list[dict[str, Any]]:
        """Fetch results from S3.

        Args:
            s3_uri: S3 URI of the results.

        Returns:
            List of result rows as dicts.
        """
        s3_client = self._get_s3_client()

        # Parse S3 URI
        if s3_uri.startswith("s3://"):
            s3_uri = s3_uri[5:]
        bucket, key = s3_uri.split("/", 1)

        try:
            response = s3_client.get_object(Bucket=bucket, Key=key)
            content = response["Body"].read().decode("utf-8")

            # Parse based on content type
            results = []
            for line in content.strip().split("\n"):
                if line:
                    try:
                        results.append(json.loads(line))
                    except json.JSONDecodeError:
                        # Plain text result
                        results.append({"output": line})
            return results

        except Exception as e:
            logger.debug(f"Could not fetch results from {s3_uri}: {e}")
            return []

    def load_data(
        self,
        tables: list[str],
        source_dir: Path | str,
        file_format: str = "parquet",
        **kwargs: Any,
    ) -> dict[str, str]:
        """Upload benchmark data to S3 and create Hive tables.

        Args:
            tables: List of table names to load.
            source_dir: Local directory containing table data files.
            file_format: Data file format (default: parquet).
            **kwargs: Additional options.

        Returns:
            Dict mapping table names to S3 URIs.
        """
        source_path = Path(source_dir)
        if not source_path.exists():
            raise ConfigurationError(f"Source directory not found: {source_dir}")

        # Check if tables already exist in S3
        if self._staging and self._staging.tables_exist(tables):
            logger.info("Tables already exist in S3 staging, skipping upload")
            return {table: self._staging.get_table_uri(table) for table in tables}

        # Upload using cloud-spark staging infrastructure
        if self._staging:
            logger.info(f"Uploading {len(tables)} tables to S3 staging")
            self._staging.upload_tables(
                tables=tables,
                source_dir=source_path,
                file_format=file_format,
            )

        # Create Hive external tables via Spark SQL
        table_uris = {}
        for table in tables:
            table_uri = f"{self.s3_staging_dir}/tables/{table}"
            table_uris[table] = table_uri

            create_table_sql = f"""
                CREATE EXTERNAL TABLE IF NOT EXISTS {self.database}.{table}
                USING PARQUET
                LOCATION '{table_uri}'
            """
            self._submit_calculation(create_table_sql, code_type="SQL", wait_for_completion=True)
            logger.info(f"Created table {self.database}.{table}")

        return table_uris

    def execute_query(
        self,
        query: str,
        **kwargs: Any,
    ) -> list[dict[str, Any]]:
        """Execute a SQL query on Athena Spark.

        Args:
            query: SQL query to execute.
            **kwargs: Additional query options.

        Returns:
            Query results as list of dicts.
        """
        start_time = mono_time()

        calculation_id, state = self._submit_calculation(query, code_type="SQL", wait_for_completion=True)

        elapsed = elapsed_seconds(start_time)
        self._query_count += 1
        self._total_execution_time_seconds += elapsed

        if state not in AthenaSparkCalculationState.SUCCESS_STATES:
            raise RuntimeError(f"Athena Spark calculation failed with state: {state}")

        # Retrieve results
        results = self._get_calculation_result(calculation_id)
        return results

    def close(self) -> None:
        """Terminate the Athena Spark session."""
        if self._session_id:
            try:
                client = self._get_athena_client()
                client.terminate_session(SessionId=self._session_id)
                logger.info(f"Terminated session: {self._session_id}")
            except Exception as e:
                logger.warning(f"Failed to terminate session: {e}")
            finally:
                self._session_id = None

        logger.info(f"Athena Spark session closed. Executed {self._query_count} calculations.")
        if self._total_execution_time_seconds > 0:
            logger.info(f"Total execution time: {self._total_execution_time_seconds:.1f}s")

    @staticmethod
    def add_cli_arguments(parser: Any) -> None:
        """Add Athena Spark-specific CLI arguments.

        Args:
            parser: Argument parser to add arguments to.
        """
        group = parser.add_argument_group("Athena Spark Options")
        group.add_argument(
            "--workgroup",
            help="Spark-enabled Athena workgroup name",
        )
        group.add_argument(
            "--s3-staging-dir",
            help="S3 path for data staging (e.g., s3://bucket/path)",
        )
        group.add_argument(
            "--region",
            default="us-east-1",
            help="AWS region (default: us-east-1)",
        )
        group.add_argument(
            "--database",
            default="benchbox",
            help="Glue Data Catalog database (default: benchbox)",
        )
        group.add_argument(
            "--coordinator-dpu-size",
            type=int,
            default=1,
            help="Coordinator DPU size (default: 1)",
        )
        group.add_argument(
            "--max-concurrent-dpus",
            type=int,
            default=20,
            help="Maximum concurrent DPUs (default: 20)",
        )
        group.add_argument(
            "--session-idle-timeout",
            type=int,
            default=15,
            help="Session idle timeout in minutes (default: 15)",
        )

    @classmethod
    def from_config(cls, config: dict[str, Any]) -> AthenaSparkAdapter:
        """Create adapter from configuration dict.

        Args:
            config: Configuration dictionary.

        Returns:
            Configured AthenaSparkAdapter instance.
        """
        params = {
            "workgroup": config.get("workgroup"),
            "s3_staging_dir": config.get("s3_staging_dir"),
            "region": config.get("region", "us-east-1"),
            "database": config.get("database", "benchbox"),
            "engine_version": config.get("engine_version", "PySpark engine version 3"),
            "session_idle_timeout_minutes": config.get("session_idle_timeout_minutes", 15),
            "coordinator_dpu_size": config.get("coordinator_dpu_size", 1),
            "max_concurrent_dpus": config.get("max_concurrent_dpus", 20),
            "default_executor_dpu_size": config.get("default_executor_dpu_size", 1),
            "timeout_minutes": config.get("timeout_minutes", 60),
        }

        return cls(**params)

    # configure_for_benchmark is inherited from CloudSparkConfigMixin

    def apply_tuning_configuration(
        self,
        config: UnifiedTuningConfiguration,
    ) -> dict[str, Any]:
        """Apply unified tuning configuration.

        Args:
            config: Unified tuning configuration.

        Returns:
            Dict with results of applied configurations.
        """
        results: dict[str, Any] = {}

        if config.scale_factor:
            self._scale_factor = config.scale_factor

        if config.primary_keys:
            results["primary_keys"] = self.apply_primary_keys(config.primary_keys)

        if config.foreign_keys:
            results["foreign_keys"] = self.apply_foreign_keys(config.foreign_keys)

        if config.platform:
            results["platform_optimizations"] = self.apply_platform_optimizations(config.platform)

        return results

    # apply_primary_keys, apply_foreign_keys, apply_platform_optimizations,
    # and apply_constraint_configuration are inherited from SparkTuningMixin

    def get_target_dialect(self) -> str:
        """Return the target SQL dialect for Athena Spark.

        Athena Spark uses Spark SQL for query execution.

        Returns:
            The dialect string "spark".
        """
        return "spark"



"""Query manager for the TPC-DS One Big Table benchmark."""

from __future__ import annotations

from typing import Any

from benchbox.core.tpcds_obt.manual_queries import MANUAL_QUERY_IDS, get_manual_query, render_manual_query
from benchbox.core.tpcds_obt.query_conversion import QueryConverter

# Queries that successfully convert to OBT schema
# Total: 89 out of 99 convertible (90% coverage)
# Blocked queries (10 total):
#   - Inventory fact table: Q21, Q22, Q37, Q39, Q72, Q82 (separate fact domain, not in OBT)
#   - Require external dimension tables for customer's CURRENT address/demographics:
#       Q46, Q64, Q68, Q84 (need customer, customer_address, household_demographics, etc.)
# Manually crafted queries: 14, 49 (complex semantics requiring manual rewrite)
# Note: Q64 uses cross-channel self-join pattern which defeats OBT's single-scan design
CONVERTIBLE_QUERY_IDS = (
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
    13,
    14,
    15,
    16,
    17,
    18,
    19,
    20,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
    38,
    40,
    41,
    42,
    43,
    44,
    45,
    47,
    48,
    49,
    50,
    51,
    52,
    53,
    54,
    55,
    56,
    57,
    58,
    59,
    60,
    61,
    62,
    63,
    65,
    66,
    67,
    69,
    70,
    71,
    73,
    74,
    75,
    76,
    77,
    78,
    79,
    80,
    81,
    83,
    85,
    86,
    87,
    88,
    89,
    90,
    91,
    92,
    93,
    94,
    95,
    96,
    97,
    98,
    99,
)


class TPCDSOBTQueryManager:
    """Generates and manages OBT-adapted TPC-DS queries."""

    def __init__(self, converter: QueryConverter | None = None) -> None:
        self.converter = converter or QueryConverter()
        self._converted = self._load_queries()
        self._manual_queries = self._load_manual_queries()

    def _load_queries(self) -> dict[int, Any]:
        queries: dict[int, Any] = {}
        for qid in CONVERTIBLE_QUERY_IDS:
            if qid in MANUAL_QUERY_IDS:
                continue  # Skip manual queries, handled separately
            queries[qid] = self.converter.convert(qid)
        return queries

    def _load_manual_queries(self) -> dict[int, Any]:
        """Load manually crafted queries for complex cases."""
        manual: dict[int, Any] = {}
        for qid in CONVERTIBLE_QUERY_IDS:
            if qid in MANUAL_QUERY_IDS:
                manual[qid] = get_manual_query(qid)
        return manual

    def get_query(self, query_id: int | str, parameters: dict[str, Any] | None = None) -> str:
        """Return rendered SQL for a specific query id."""
        qid_int = self._normalize_id(query_id)
        if qid_int in self._manual_queries:
            return render_manual_query(qid_int, parameters)
        converted = self._get_converted(query_id)
        return self._render_query(converted, parameters or {})

    def get_template(self, query_id: int | str) -> str:
        """Return the template SQL with parameter placeholders intact."""
        qid_int = self._normalize_id(query_id)
        if qid_int in self._manual_queries:
            return self._manual_queries[qid_int].template_sql
        return self._get_converted(query_id).template_sql

    def get_queries(self, parameters: dict[str, Any] | None = None) -> dict[int, str]:
        """Return all rendered SQL queries keyed by numeric id."""
        return {qid: self.get_query(qid, parameters) for qid in self.list_query_ids()}

    def list_query_ids(self) -> list[int]:
        """Return available query identifiers."""
        all_ids = set(self._converted.keys()) | set(self._manual_queries.keys())
        return sorted(all_ids)

    def _render_query(self, converted: Any, parameters: dict[str, Any]) -> str:
        if not parameters:
            return converted.default_sql
        sql = converted.template_sql
        for name, param in converted.parameters.items():
            value = parameters.get(name, param.default)
            replacement = param.render(value)
            sql = self.converter._param_pattern(name).sub(replacement, sql)  # noqa: SLF001
        return sql

    def _get_converted(self, query_id: int | str) -> Any:
        qid_int = self._normalize_id(query_id)
        if qid_int in self._manual_queries:
            raise ValueError(f"Query {query_id} is a manual query, use get_query() instead")
        if qid_int not in self._converted:
            raise ValueError(f"Unknown query id: {query_id}")
        return self._converted[qid_int]

    def _normalize_id(self, query_id: int | str) -> int:
        if isinstance(query_id, int):
            return query_id
        try:
            return int(query_id)
        except ValueError as exc:
            raise ValueError(f"Unknown query id: {query_id}") from exc


__all__ = ["CONVERTIBLE_QUERY_IDS", "TPCDSOBTQueryManager"]



"""Configuration management for BenchBox CLI.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

import argparse
import json
import os
from datetime import datetime
from pathlib import Path
from typing import Any, Optional, Union

import yaml
from pydantic import BaseModel, ConfigDict, Field

from benchbox.core.config_utils import (
    deep_merge_dicts,
    load_config_file,
    save_config_file,
)
from benchbox.core.tuning.interface import (
    BenchmarkTunings,
    TableTuning,
    TuningColumn,
    TuningType,
    UnifiedTuningConfiguration,
)
from benchbox.utils.database_naming import generate_database_filename
from benchbox.utils.printing import quiet_console
from benchbox.utils.scale_factor import format_scale_factor

console = quiet_console


def load_config(
    cli_args: Optional[dict[str, Any]] = None,
    config_file: Optional[Path] = None,
    validate: bool = True,
) -> "BenchBoxConfig":
    """Unified configuration loader with explicit precedence.

    Loads configuration with the following precedence (highest to lowest):
    1. CLI arguments (cli_args parameter)
    2. Environment variables (BENCHBOX_* prefixed)
    3. Configuration file (YAML/JSON)
    4. Default values

    Args:
        cli_args: Dictionary of CLI argument overrides
        config_file: Path to configuration file (optional, will search default locations)
        validate: Whether to validate the configuration after loading

    Returns:
        Loaded and validated BenchBoxConfig instance

    Example:
        >>> # Load with defaults
        >>> config = load_config()
        >>>
        >>> # Load with CLI overrides
        >>> config = load_config(cli_args={'database': {'preferred': 'clickhouse'}})
        >>>
        >>> # Load from specific file
        >>> config = load_config(config_file=Path('my_config.yaml'))
    """
    # Create ConfigManager to handle file loading
    manager = ConfigManager(config_path=config_file)

    # Start with file-based config (already has defaults merged)
    config_dict = manager.config.model_dump()

    # Apply environment variable overrides
    config_dict = _apply_environment_overrides(config_dict)

    # Apply CLI argument overrides (highest priority)
    if cli_args:
        config_dict = deep_merge_dicts(config_dict, cli_args)

    # Create validated config
    config = BenchBoxConfig(**config_dict)

    # Validate if requested
    if validate:
        manager.config = config
        if not manager.validate_config():
            raise ValueError("Configuration validation failed")

    return config


def _apply_environment_overrides(config_dict: dict[str, Any]) -> dict[str, Any]:
    """Apply environment variable overrides to configuration.

    Supports the following environment variables:
    - BENCHBOX_DATABASE_PREFERRED: Override database.preferred
    - BENCHBOX_SCALE_FACTOR: Override benchmarks.default_scale
    - BENCHBOX_VERBOSE: Override execution.verbose
    - BENCHBOX_MAX_WORKERS: Override execution.max_workers
    - BENCHBOX_TUNING_ENABLED: Override tuning.enabled
    - BENCHBOX_TUNING_CONFIG: Override tuning.default_config_file

    Args:
        config_dict: Configuration dictionary

    Returns:
        Configuration dictionary with environment overrides applied
    """
    # Define environment variable mappings
    env_mappings = {
        "BENCHBOX_DATABASE_PREFERRED": ("database", "preferred", str),
        "BENCHBOX_SCALE_FACTOR": ("benchmarks", "default_scale", float),
        "BENCHBOX_VERBOSE": ("execution", "verbose", lambda v: v.lower() in ["true", "1", "yes", "on"]),
        "BENCHBOX_MAX_WORKERS": ("execution", "max_workers", int),
        "BENCHBOX_TUNING_ENABLED": ("tuning", "enabled", lambda v: v.lower() in ["true", "1", "yes", "on"]),
        "BENCHBOX_TUNING_CONFIG": ("tuning", "default_config_file", str),
        "BENCHBOX_OUTPUT_DIR": ("output", "directory", str),
        "BENCHBOX_MEMORY_LIMIT_GB": ("execution", "memory_limit_gb", int),
    }

    for env_var, (section, key, converter) in env_mappings.items():
        env_value = os.getenv(env_var)
        if env_value is not None:
            try:
                converted_value = converter(env_value)
                if section not in config_dict:
                    config_dict[section] = {}
                config_dict[section][key] = converted_value
            except (ValueError, TypeError) as e:
                console.print(f"[yellow]Warning: Invalid environment variable {env_var}={env_value}: {e}[/yellow]")

    return config_dict


class BenchBoxConfig(BaseModel):
    """Main configuration model."""

    model_config = ConfigDict(extra="allow")

    system: dict[str, Any] = Field(default_factory=dict)
    database: dict[str, Any] = Field(default_factory=dict)
    benchmarks: dict[str, Any] = Field(default_factory=dict)
    output: dict[str, Any] = Field(default_factory=dict)
    execution: dict[str, Any] = Field(default_factory=dict)
    tuning: dict[str, Any] = Field(default_factory=dict)


class ConfigManager:
    """Configuration file and settings management."""

    def __init__(self, config_path: Optional[Path] = None):
        self.console = quiet_console
        self.config_path = config_path or self._get_default_config_path()
        self.config = self._load_config()
        # Apply environment variable overrides for tuning settings
        self.apply_environment_overrides()

    def _get_default_config_path(self) -> Path:
        """Get default configuration file path."""
        # Check for config in current directory first
        current_config = Path("benchbox.yaml")
        if current_config.exists():
            return current_config

        # Check user home directory
        home_config = Path.home() / ".benchbox" / "config.yaml"
        return home_config

    def _load_config(self) -> BenchBoxConfig:
        """Load configuration from file or create default."""
        try:
            if self.config_path.exists():
                try:
                    with open(self.config_path) as f:
                        config_data = yaml.safe_load(f) or {}

                    # If config file is empty or doesn't have any of our main sections,
                    # return default config
                    if not config_data or not any(
                        key in config_data
                        for key in [
                            "system",
                            "database",
                            "benchmarks",
                            "output",
                            "execution",
                        ]
                    ):
                        return self._get_default_config()

                    # Merge with defaults to ensure all required fields exist
                    default_config = self._get_default_config()
                    merged_config = deep_merge_dicts(default_config.model_dump(), config_data)
                    return BenchBoxConfig(**merged_config)
                except Exception as e:
                    console.print(f"[yellow]Warning: Failed to load config from {self.config_path}: {e}[/yellow]")
                    return self._get_default_config()
            else:
                return self._get_default_config()
        except (PermissionError, OSError) as e:
            console.print(f"[yellow]Warning: Permission denied accessing {self.config_path}: {e}[/yellow]")
            return self._get_default_config()

    def _get_default_config(self) -> BenchBoxConfig:
        """Get default configuration."""
        return BenchBoxConfig(
            system={
                "auto_profile": True,
                "save_profile": True,
                "profile_cache_hours": 24,
            },
            database={
                "preferred": "duckdb",
                "connection_timeout": 30,
                "auto_detect": True,
            },
            benchmarks={
                "default_scale": 0.01,
                "timeout_minutes": 60,
                "max_memory_gb": 8,
                "continue_on_error": False,
            },
            output={
                "formats": ["json", "console"],
                "directory": "./benchmark_runs/results",
                "timestamp_format": "%Y%m%d_%H%M%S",
                "submit_to_service": False,
                "service_url": "https://api.benchbox.dev/v1",
                "compression": {
                    "enabled": True,
                    "type": "zstd",
                    "level": None,  # Use algorithm defaults
                },
            },
            execution={
                "parallel_queries": False,
                "max_workers": 4,
                "memory_limit_gb": 0,  # 0 = auto
                "verbose": True,
                "power_run": {
                    "iterations": 3,  # Changed from 1 to 3: run 3 measurement iterations
                    "warm_up_iterations": 1,
                    "timeout_per_iteration_minutes": 60,
                    "fail_fast": False,
                    "collect_metrics": True,
                },
                "concurrent_queries": {
                    "enabled": False,
                    "max_concurrent": 2,
                    "query_timeout_seconds": 300,
                    "stream_timeout_seconds": 3600,
                    "retry_failed_queries": True,
                    "max_retries": 3,
                },
            },
            tuning={
                "enabled": False,
                "default_config_file": None,
                "validate_on_load": True,
                "allow_platform_incompatible": False,
                "environment_overrides": {
                    "BENCHBOX_TUNING_ENABLED": "enabled",
                    "BENCHBOX_TUNING_CONFIG": "default_config_file",
                },
            },
        )

    def save_config(self):
        """Save current configuration to file."""
        try:
            # Create directory if it doesn't exist
            self.config_path.parent.mkdir(parents=True, exist_ok=True)

            # Convert to dict and save
            config_dict = self.config.model_dump()

            # Add metadata
            config_dict["_metadata"] = {
                "version": "1.0",
                "created": datetime.now().isoformat(),
                "generated_by": "benchbox-cli",
            }

            with open(self.config_path, "w") as f:
                yaml.dump(config_dict, f, default_flow_style=False, sort_keys=False)

            console.print(f"[green]✅ Configuration saved to {self.config_path}[/green]")
        except Exception as e:
            console.print(f"[red]❌ Failed to save configuration: {e}[/red]")

    def get(self, key: str, default: Any = None) -> Any:
        """Get configuration value using dot notation."""
        keys = key.split(".")
        value = self.config.model_dump()

        for k in keys:
            if isinstance(value, dict) and k in value:
                value = value[k]
            else:
                return default

        return value

    def set(self, key: str, value: Any):
        """Set configuration value using dot notation."""
        keys = key.split(".")
        config_dict = self.config.model_dump()
        current = config_dict

        for k in keys[:-1]:
            if k not in current:
                current[k] = {}
            current = current[k]

        current[keys[-1]] = value
        self.config = BenchBoxConfig(**config_dict)

    def update_from_system_profile(self, profile):
        """Update configuration based on system profile."""
        self.set("system.detected_os", profile.os_name)
        self.set("system.detected_arch", profile.architecture)
        self.set("system.detected_memory_gb", profile.memory_total_gb)
        self.set("system.detected_cpu_cores", profile.cpu_cores_logical)
        self.set("system.available_databases", profile.available_databases)
        self.set("system.last_profile_time", profile.timestamp.isoformat())

        # Auto-configure based on system
        if profile.memory_total_gb > 0:
            # Set memory limit to 75% of available memory
            memory_limit = max(1, int(profile.memory_total_gb * 0.75))
            self.set("benchmarks.max_memory_gb", memory_limit)
            self.set("execution.memory_limit_gb", memory_limit)

        # Set max workers based on CPU cores
        max_workers = min(8, max(2, profile.cpu_cores_logical // 2))
        self.set("execution.max_workers", max_workers)

        # Auto-configure concurrent queries based on system resources
        concurrent_max = min(max_workers, max(2, profile.cpu_cores_logical // 4))
        self.set("execution.concurrent_queries.max_concurrent", concurrent_max)

        # Scale timeouts based on available memory (lower memory = longer timeouts)
        if profile.memory_total_gb < 8:
            # Low memory systems need longer timeouts
            self.set("execution.power_run.timeout_per_iteration_minutes", 120)
            self.set("execution.concurrent_queries.query_timeout_seconds", 600)
        elif profile.memory_total_gb > 16:
            # High memory systems can use shorter timeouts
            self.set("execution.power_run.timeout_per_iteration_minutes", 45)
            self.set("execution.concurrent_queries.query_timeout_seconds", 180)

        # Prefer DuckDB if available
        if "duckdb" in profile.available_databases:
            self.set("database.preferred", "duckdb")
        elif profile.available_databases:
            self.set("database.preferred", profile.available_databases[0])

    def validate_config(self) -> bool:
        """Validate current configuration."""
        try:
            # Basic validation
            if self.get("benchmarks.default_scale", 0) <= 0:
                console.print("[red]❌ Invalid default scale factor[/red]")
                return False

            if self.get("benchmarks.timeout_minutes", 0) <= 0:
                console.print("[red]❌ Invalid timeout value[/red]")
                return False

            if self.get("execution.max_workers", 0) <= 0:
                console.print("[red]❌ Invalid max workers value[/red]")
                return False

            # Power run validation
            power_iterations = self.get("execution.power_run.iterations", 4)
            if power_iterations <= 0:
                console.print("[red]❌ Invalid power run iterations value[/red]")
                return False

            power_warm_up = self.get("execution.power_run.warm_up_iterations", 0)
            if power_warm_up < 0:
                console.print("[red]❌ Invalid power run warm-up iterations value[/red]")
                return False

            power_timeout = self.get("execution.power_run.timeout_per_iteration_minutes", 60)
            if power_timeout <= 0:
                console.print("[red]❌ Invalid power run timeout value[/red]")
                return False

            # Concurrent queries validation
            max_concurrent = self.get("execution.concurrent_queries.max_concurrent", 2)
            if max_concurrent <= 0:
                console.print("[red]❌ Invalid max concurrent queries value[/red]")
                return False

            query_timeout = self.get("execution.concurrent_queries.query_timeout_seconds", 300)
            if query_timeout <= 0:
                console.print("[red]❌ Invalid concurrent query timeout value[/red]")
                return False

            stream_timeout = self.get("execution.concurrent_queries.stream_timeout_seconds", 3600)
            if stream_timeout <= 0:
                console.print("[red]❌ Invalid concurrent stream timeout value[/red]")
                return False

            max_retries = self.get("execution.concurrent_queries.max_retries", 3)
            if max_retries < 0:
                console.print("[red]❌ Invalid max retries value[/red]")
                return False

            console.print("[green]✅ Configuration validation passed[/green]")
            return True
        except Exception as e:
            console.print(f"[red]❌ Configuration validation failed: {e}[/red]")
            return False

    def show_config(self):
        """Display current configuration."""
        from rich.syntax import Syntax

        config_yaml = yaml.dump(self.config.model_dump(), default_flow_style=False, sort_keys=False)
        syntax = Syntax(config_yaml, "yaml", theme="monokai", line_numbers=True)

        console.print("\n[bold]Current Configuration:[/bold]")
        console.print(f"Config file: [cyan]{self.config_path}[/cyan]")
        console.print(syntax)

    def create_sample_config(self, path: Optional[Path] = None):
        """Create a sample configuration file."""
        if path is None:
            path = Path("benchbox.yaml")

        sample_config = self._get_default_config()

        # Add comments for better understanding
        config_dict = sample_config.model_dump()
        config_dict["_comments"] = {
            "system": "System profiling and detection settings",
            "database": "Database connection and preference settings",
            "benchmarks": "Default benchmark execution parameters",
            "output": "Result formatting and export settings",
            "execution": "Performance and execution control settings",
            "execution.power_run": "Configuration for power run test iterations and metrics collection",
            "execution.concurrent_queries": "Configuration for concurrent query execution and throughput testing",
            "tuning": "Table tuning and optimization settings",
        }

        with open(path, "w") as f:
            yaml.dump(config_dict, f, default_flow_style=False, sort_keys=False)

        console.print(f"[green]✅ Sample configuration created at {path}[/green]")
        console.print("Edit this file to customize BenchBox behavior.")

    def load_tuning_config(self, config_path: Union[str, Path]) -> dict[str, BenchmarkTunings]:
        """Load tuning configuration from YAML or JSON file.

        Args:
            config_path: Path to the tuning configuration file

        Returns:
            Dictionary mapping benchmark names to their tuning configurations

        Raises:
            ValueError: If the configuration file is invalid or cannot be loaded
        """
        config_path = Path(config_path)

        try:
            config_data = load_config_file(config_path)
            if not config_data:
                raise ValueError("Configuration file is empty")

            # Parse tuning configurations
            benchmark_tunings = {}

            for benchmark_name, benchmark_data in config_data.items():
                # Skip metadata sections
                if benchmark_name.startswith("_"):
                    continue

                if not isinstance(benchmark_data, dict):
                    raise ValueError(f"Invalid benchmark configuration for '{benchmark_name}': must be a dictionary")

                # Create BenchmarkTunings object
                tunings = BenchmarkTunings(benchmark_name=benchmark_name)

                # Parse table tunings
                for table_name, table_data in benchmark_data.items():
                    if table_name.startswith("_"):  # Skip metadata fields
                        continue

                    table_tuning = self._parse_table_tuning(table_name, table_data)
                    tunings.add_table_tuning(table_tuning)

                benchmark_tunings[benchmark_name] = tunings

            # Validate configurations if enabled
            if self.get("tuning.validate_on_load", True):
                self._validate_tuning_configs(benchmark_tunings)

            return benchmark_tunings

        except (yaml.YAMLError, json.JSONDecodeError) as e:
            raise ValueError(f"Failed to parse tuning configuration file: {e}")
        except Exception as e:
            raise ValueError(f"Error loading tuning configuration: {e}")

    def _parse_table_tuning(self, table_name: str, table_data: dict[str, Any]) -> TableTuning:
        """Parse table tuning configuration from dictionary data.

        Args:
            table_name: Name of the table
            table_data: Dictionary containing tuning configuration

        Returns:
            TableTuning object
        """
        # Parse optional tuning type columns
        partitioning: Optional[list[TuningColumn]] = None
        clustering: Optional[list[TuningColumn]] = None
        distribution: Optional[list[TuningColumn]] = None
        sorting: Optional[list[TuningColumn]] = None

        for tuning_type_str in [
            "partitioning",
            "clustering",
            "distribution",
            "sorting",
        ]:
            if tuning_type_str in table_data:
                columns_data = table_data[tuning_type_str]
                if not isinstance(columns_data, list):
                    raise ValueError(f"'{tuning_type_str}' must be a list of column configurations")

                columns = []
                for i, col_data in enumerate(columns_data):
                    if isinstance(col_data, str):
                        # Simple format: just column name, infer order
                        columns.append(TuningColumn(name=col_data, type="UNKNOWN", order=i + 1))
                    elif isinstance(col_data, dict):
                        # Full format with name, type, and order
                        columns.append(TuningColumn.from_dict(col_data))
                    else:
                        raise ValueError(f"Invalid column configuration at index {i} in {tuning_type_str}")

                # Assign to the appropriate variable
                if tuning_type_str == "partitioning":
                    partitioning = columns
                elif tuning_type_str == "clustering":
                    clustering = columns
                elif tuning_type_str == "distribution":
                    distribution = columns
                elif tuning_type_str == "sorting":
                    sorting = columns

        # Create TableTuning with explicit parameters (helps type checker)
        return TableTuning(
            table_name=table_name,
            partitioning=partitioning,
            clustering=clustering,
            distribution=distribution,
            sorting=sorting,
        )

    def _validate_tuning_configs(self, benchmark_tunings: dict[str, BenchmarkTunings]) -> None:
        """Validate loaded tuning configurations.

        Args:
            benchmark_tunings: Dictionary of benchmark tuning configurations
        """
        for benchmark_name, tunings in benchmark_tunings.items():
            if not tunings.has_valid_tunings():
                validation_results = tunings.validate_all()

                error_messages = []
                for table_name, errors in validation_results.items():
                    if errors:
                        error_messages.extend([f"{table_name}: {error}" for error in errors])

                if error_messages:
                    console.print(f"[red]⚠️ Tuning validation errors in '{benchmark_name}':[/red]")
                    for msg in error_messages:
                        console.print(f"  - {msg}")

                    if not self.get("tuning.allow_platform_incompatible", False):
                        raise ValueError(f"Invalid tuning configuration for benchmark '{benchmark_name}'")

    def save_tuning_config(
        self,
        benchmark_tunings: dict[str, BenchmarkTunings],
        config_path: Union[str, Path],
        format: str = "yaml",
    ) -> None:
        """Save tuning configurations to file.

        Args:
            benchmark_tunings: Dictionary of benchmark tuning configurations
            config_path: Path where to save the configuration
            format: File format ('yaml' or 'json')
        """
        config_path = Path(config_path)

        # Convert to serializable format
        config_data = {}
        for benchmark_name, tunings in benchmark_tunings.items():
            config_data[benchmark_name] = tunings.to_dict()["table_tunings"]

        # Add metadata
        config_data["_metadata"] = {
            "version": "1.0",
            "created": datetime.now().isoformat(),
            "generated_by": "benchbox-cli",
        }

        # Save to file
        try:
            save_config_file(config_data, config_path, format)
            console.print(f"[green]✅ Tuning configuration saved to {config_path}[/green]")

        except Exception as e:
            console.print(f"[red]❌ Failed to save tuning configuration: {e}[/red]")
            raise

    def apply_environment_overrides(self) -> None:
        """Apply environment variable overrides for tuning settings."""
        env_overrides = self.get("tuning.environment_overrides", {})

        for env_var, config_key in env_overrides.items():
            env_value = os.getenv(env_var)
            if env_value is not None:
                # Convert environment variable values to appropriate types
                if config_key == "enabled":
                    value = env_value.lower() in ["true", "1", "yes", "on", "enabled"]
                    self.set(f"tuning.{config_key}", value)
                else:
                    self.set(f"tuning.{config_key}", env_value)

                console.print(f"[yellow]Applied environment override: {env_var}={env_value}[/yellow]")

    def load_unified_tuning_config(
        self, config_path: Union[str, Path], platform: Optional[str] = None
    ) -> UnifiedTuningConfiguration:
        """Load unified tuning configuration from YAML or JSON file.

        Args:
            config_path: Path to the unified tuning configuration file
            platform: Platform to validate against (default: uses configured preferred platform)

        Returns:
            UnifiedTuningConfiguration instance

        Raises:
            ValueError: If the configuration file is invalid or cannot be loaded
        """
        config_path = Path(config_path)

        try:
            config_data = load_config_file(config_path)
            if not config_data:
                raise ValueError("Configuration file is empty")

            # Create unified configuration from data
            unified_config = UnifiedTuningConfiguration.from_dict(config_data)

            # Validate configuration if enabled
            if self.get("tuning.validate_on_load", True):
                self._validate_unified_tuning_config(unified_config, platform)

            return unified_config

        except (yaml.YAMLError, json.JSONDecodeError) as e:
            raise ValueError(f"Failed to parse unified tuning configuration file: {e}")
        except Exception as e:
            raise ValueError(f"Error loading unified tuning configuration: {e}")

    def save_unified_tuning_config(
        self,
        config: UnifiedTuningConfiguration,
        config_path: Union[str, Path],
        format: str = "yaml",
    ) -> None:
        """Save unified tuning configuration to file.

        Args:
            config: UnifiedTuningConfiguration to save
            config_path: Path where to save the configuration
            format: File format ('yaml' or 'json')
        """
        config_path = Path(config_path)

        # Convert to serializable format
        config_data = config.to_dict()

        # Add metadata
        config_data["_metadata"] = {
            "version": "2.0",
            "format": "unified_tuning",
            "created": datetime.now().isoformat(),
            "generated_by": "benchbox-cli",
        }

        # Save to file
        try:
            save_config_file(config_data, config_path, format)
            console.print(f"[green]✅ Unified tuning configuration saved to {config_path}[/green]")

        except Exception as e:
            console.print(f"[red]❌ Failed to save unified tuning configuration: {e}[/red]")
            raise

    def _validate_unified_tuning_config(
        self, config: UnifiedTuningConfiguration, platform: Optional[str] = None
    ) -> None:
        """Validate unified tuning configuration.

        Args:
            config: UnifiedTuningConfiguration to validate
            platform: Platform to validate against (default: uses configured preferred platform)
        """
        if platform is None:
            platform = self.get("database.preferred", "duckdb")

        # Validate constraint settings are explicitly specified
        constraint_errors = []

        if config.primary_keys.enabled is None:
            constraint_errors.append("primary_keys.enabled must be explicitly specified (true or false)")

        if config.foreign_keys.enabled is None:
            constraint_errors.append("foreign_keys.enabled must be explicitly specified (true or false)")

        # Validate platform-specific configuration
        platform_errors = config.validate_for_platform(platform)

        all_errors = constraint_errors + platform_errors

        if all_errors:
            console.print(f"[red]⚠️ Unified tuning validation errors for platform '{platform}':[/red]")
            for error in all_errors:
                console.print(f"  - {error}")

            if not self.get("tuning.allow_platform_incompatible", False):
                raise ValueError(f"Invalid unified tuning configuration for platform '{platform}'")

    def create_sample_unified_tuning_config(self, path: Optional[Path] = None, platform: Optional[str] = None) -> None:
        """Create a sample unified tuning configuration file.

        Args:
            path: Path where to create the sample configuration (default: unified_tuning.yaml)
            platform: Target platform for compatibility (default: uses configured preferred platform)
        """
        if path is None:
            path = Path("unified_tuning.yaml")

        if platform is None:
            platform = self.get("database.preferred", "duckdb")

        # Create a sample configuration with platform-compatible options
        sample_config = UnifiedTuningConfiguration()

        # Configure constraints (supported by most platforms)
        sample_config.primary_keys.enabled = True
        sample_config.foreign_keys.enabled = True
        sample_config.foreign_keys.on_delete_action = "CASCADE"

        sample_config.unique_constraints.enabled = True
        sample_config.check_constraints.enabled = True

        # Configure platform-specific optimizations based on target platform
        if platform.lower() == "databricks":
            sample_config.platform_optimizations.z_ordering_enabled = True
            sample_config.platform_optimizations.z_ordering_columns = [
                "date_col",
                "partition_col",
            ]
            sample_config.platform_optimizations.databricks_clustering_strategy = "z_order"
            sample_config.platform_optimizations.liquid_clustering_enabled = False
            sample_config.platform_optimizations.liquid_clustering_columns = []
            sample_config.platform_optimizations.sorted_ingestion_mode = "off"
            sample_config.platform_optimizations.sorted_ingestion_method = "auto"
            sample_config.platform_optimizations.auto_optimize_enabled = True
            sample_config.platform_optimizations.bloom_filters_enabled = True
            sample_config.platform_optimizations.bloom_filter_columns = [
                "customer_id",
                "product_id",
            ]
        elif platform.lower() in ["snowflake", "bigquery"]:
            sample_config.platform_optimizations.materialized_views_enabled = True
        elif platform.lower() in ["postgresql", "redshift"]:
            sample_config.platform_optimizations.bloom_filters_enabled = True
            sample_config.platform_optimizations.bloom_filter_columns = ["customer_id"]
            sample_config.platform_optimizations.materialized_views_enabled = True

        # Add platform-compatible table tunings
        from benchbox.core.tuning.interface import TableTuning, TuningColumn

        if TuningType.PARTITIONING.is_compatible_with_platform(platform):
            sample_config.table_tunings["orders"] = TableTuning(
                table_name="orders",
                partitioning=[TuningColumn(name="order_date", type="DATE", order=1)],
            )

            sample_config.table_tunings["lineitem"] = TableTuning(
                table_name="lineitem",
                partitioning=[TuningColumn(name="shipdate", type="DATE", order=1)],
            )

        if TuningType.CLUSTERING.is_compatible_with_platform(platform):
            if "orders" in sample_config.table_tunings:
                sample_config.table_tunings["orders"].clustering = [
                    TuningColumn(name="customer_id", type="INTEGER", order=1)
                ]

        if TuningType.SORTING.is_compatible_with_platform(platform):
            if "lineitem" not in sample_config.table_tunings:
                sample_config.table_tunings["lineitem"] = TableTuning(table_name="lineitem")

            sample_config.table_tunings["lineitem"].sorting = [
                TuningColumn(name="orderkey", type="INTEGER", order=1),
                TuningColumn(name="linenumber", type="INTEGER", order=2),
            ]

        self.save_unified_tuning_config(sample_config, path)
        console.print(
            f"[green]✅ Sample unified tuning configuration created for platform '{platform}' at {path}[/green]"
        )

        enabled_types = sample_config.get_enabled_tuning_types()
        console.print("This configuration includes:")
        console.print("  • Schema constraints (primary keys, foreign keys, unique, check)")
        if any(t.value.startswith(("z_ordering", "auto_", "bloom_", "materialized_")) for t in enabled_types):
            console.print(f"  • Platform optimizations for {platform}")
        if any(
            t
            in [
                TuningType.PARTITIONING,
                TuningType.CLUSTERING,
                TuningType.SORTING,
                TuningType.DISTRIBUTION,
            ]
            for t in enabled_types
        ):
            console.print("  • Table-level tunings (partitioning, clustering, sorting)")
        console.print(f"All settings are compatible with {platform}.")
        console.print("Edit this file to customize your tuning settings.")


class DirectoryManager:
    """Manages BenchBox directory structure and file organization."""

    def __init__(self, base_dir: Optional[str] = None):
        """Initialize directory manager with configurable base directory."""
        self.base_dir = Path(base_dir or "benchmark_runs")
        self.results_dir = self.base_dir / "results"
        self.datagen_dir = self.base_dir / "datagen"
        self.databases_dir = self.base_dir / "databases"

        # Create directories if they don't exist
        self._ensure_directories()

    def _ensure_directories(self):
        """Create all required directories."""
        for directory in [
            self.base_dir,
            self.results_dir,
            self.datagen_dir,
            self.databases_dir,
        ]:
            directory.mkdir(parents=True, exist_ok=True)

    def _format_scale_factor(self, scale_factor: float) -> str:
        """Format scale factor for filenames using centralized utility."""
        return format_scale_factor(scale_factor)

    def get_result_filename(
        self,
        benchmark_name: str,
        scale_factor: float,
        platform: str,
        timestamp: str,
        execution_id: str,
        mode: str | None = None,
    ) -> str:
        """Generate standardized result filename."""
        from benchbox.core.results.filenames import build_result_filename

        return build_result_filename(
            benchmark_id=benchmark_name,
            scale_factor=scale_factor,
            platform=platform,
            timestamp=timestamp,
            execution_id=execution_id,
            mode=mode,
        )

    def get_result_path(
        self,
        benchmark_name: str,
        scale_factor: float,
        platform: str,
        timestamp: str,
        execution_id: str,
        mode: str | None = None,
    ) -> Path:
        """Get full path for result file."""
        filename = self.get_result_filename(benchmark_name, scale_factor, platform, timestamp, execution_id, mode=mode)
        return self.results_dir / filename

    def get_database_filename(
        self,
        benchmark_name: str,
        scale_factor: float,
        platform: str,
        tuning_config: Optional[dict[str, Any]] = None,
        custom_name: Optional[str] = None,
    ) -> str:
        """Generate database filename with configuration characteristics.

        Args:
            benchmark_name: Name of the benchmark
            scale_factor: Scale factor value
            platform: Platform name
            tuning_config: Unified tuning configuration (optional)
            custom_name: Custom database name override (optional)

        Returns:
            Database filename with appropriate extension
        """
        return generate_database_filename(
            benchmark_name=benchmark_name,
            scale_factor=scale_factor,
            platform=platform,
            tuning_config=tuning_config,
            custom_name=custom_name,
        )

    def get_database_path(
        self,
        benchmark_name: str,
        scale_factor: float,
        platform: str,
        tuning_config: Optional[dict[str, Any]] = None,
        custom_name: Optional[str] = None,
    ) -> Path:
        """Get full path for database file with configuration characteristics.

        Args:
            benchmark_name: Name of the benchmark
            scale_factor: Scale factor value
            platform: Platform name
            tuning_config: Unified tuning configuration (optional)
            custom_name: Custom database name override (optional)

        Returns:
            Full path to database file
        """
        filename = self.get_database_filename(benchmark_name, scale_factor, platform, tuning_config, custom_name)
        return self.databases_dir / filename

    def get_datagen_path(self, benchmark_name: str, scale_factor: float) -> Path:
        """Get path for generated data directory."""
        sf_str = self._format_scale_factor(scale_factor)
        return self.datagen_dir / f"{benchmark_name}_{sf_str}"

    def clean_old_files(self, benchmark_name: Optional[str] = None, max_age_days: int = 30):
        """Clean old files from all directories."""
        import time

        current_time = time.time()
        cutoff_time = current_time - (max_age_days * 24 * 60 * 60)

        cleaned_files = []

        for directory in [self.results_dir, self.datagen_dir, self.databases_dir]:
            if not directory.exists():
                continue

            for file_path in directory.rglob("*"):
                if file_path.is_file():
                    # Check if file matches benchmark filter
                    if benchmark_name and not file_path.name.startswith(benchmark_name):
                        continue

                    # Check file age
                    if file_path.stat().st_mtime < cutoff_time:
                        try:
                            file_path.unlink()
                            cleaned_files.append(str(file_path))
                        except Exception as e:
                            console.print(f"[yellow]Warning: Could not delete {file_path}: {e}[/yellow]")

        return cleaned_files

    def list_files(self, file_type: str = "all") -> dict[str, list[Path]]:
        """List files by type."""
        files = {
            "results": list(self.results_dir.glob("*.json")) if self.results_dir.exists() else [],
            "databases": list(self.databases_dir.glob("*")) if self.databases_dir.exists() else [],
            "datagen": list(self.datagen_dir.glob("*")) if self.datagen_dir.exists() else [],
        }

        if file_type == "all":
            return files
        else:
            return {file_type: files.get(file_type, [])}

    def get_directory_sizes(self) -> dict[str, float]:
        """Get size of each directory in MB."""

        def get_dir_size(path: Path) -> float:
            if not path.exists():
                return 0.0
            total = 0
            for file_path in path.rglob("*"):
                if file_path.is_file():
                    total += file_path.stat().st_size
            return total / (1024 * 1024)  # Convert to MB

        return {
            "results": get_dir_size(self.results_dir),
            "datagen": get_dir_size(self.datagen_dir),
            "databases": get_dir_size(self.databases_dir),
            "total": get_dir_size(self.base_dir),
        }


class ExampleArgumentParser:
    """Reusable argument parser for examples with standard patterns.

    Consolidates repetitive argparse patterns found across example files
    to provide consistent command-line interfaces and reduce duplication.
    """

    @classmethod
    def create_benchmark_parser(cls, description: str, default_scale: float = 1.0) -> argparse.ArgumentParser:
        """Create parser with common benchmark arguments.

        Args:
            description: Parser description
            default_scale: Default scale factor value

        Returns:
            ArgumentParser with standard benchmark arguments
        """
        parser = argparse.ArgumentParser(
            description=description,
            formatter_class=argparse.RawDescriptionHelpFormatter,
        )

        # Core benchmark arguments
        parser.add_argument(
            "--scale",
            type=float,
            default=default_scale,
            help=f"Scale factor (default: {default_scale})",
        )
        parser.add_argument(
            "--platform",
            type=str,
            help="Database file path or connection string (default: auto-generated)",
        )
        parser.add_argument(
            "--output",
            type=str,
            help="Output directory for data files (default: auto-generated)",
        )

        # Execution control
        parser.add_argument("--verbose", action="store_true", help="Show detailed execution information")
        parser.add_argument(
            "--quiet",
            action="store_true",
            help="Minimize output (only show essential information)",
        )

        return parser

    @classmethod
    def add_tuning_arguments(cls, parser: argparse.ArgumentParser) -> None:
        """Add standard tuning-related arguments.

        Args:
            parser: ArgumentParser to add tuning arguments to
        """
        tuning_group = parser.add_argument_group("Tuning Options", "Database optimization and constraint management")

        tuning_group.add_argument(
            "--tuning-config",
            type=str,
            metavar="FILE",
            help="Load unified tuning configuration from YAML file",
        )
        tuning_group.add_argument(
            "--create-sample-tuning",
            type=str,
            metavar="FILE",
            help="Create a sample tuning configuration file and exit",
        )
        tuning_group.add_argument(
            "--platform-name",
            type=str,
            dest="database_name",
            help="Custom database name (overrides auto-generated naming)",
        )
        tuning_group.add_argument(
            "--force",
            action="store_true",
            help="Force regeneration of data and recreation of database objects",
        )

    @classmethod
    def add_execution_arguments(cls, parser: argparse.ArgumentParser) -> None:
        """Add execution-related arguments for power runs and concurrency.

        Args:
            parser: ArgumentParser to add execution arguments to
        """
        # Power run arguments
        power_group = parser.add_argument_group("Power Run Options", "Multiple iterations with statistical analysis")

        power_group.add_argument(
            "--iterations",
            type=int,
            default=4,
            help="Number of benchmark iterations (default: 4)",
        )
        power_group.add_argument(
            "--warm-up",
            type=int,
            default=0,
            help="Number of warm-up iterations (default: 0)",
        )
        power_group.add_argument(
            "--profile",
            choices=["quick", "standard", "thorough", "stress"],
            help="Performance testing profile",
        )

        # Concurrent execution arguments
        concurrent_group = parser.add_argument_group("Concurrent Queries", "Throughput testing with multiple streams")

        concurrent_group.add_argument(
            "--concurrent",
            action="store_true",
            help="Enable concurrent query execution",
        )
        concurrent_group.add_argument(
            "--streams",
            type=int,
            default=2,
            help="Number of concurrent query streams (default: 2)",
        )

        # Phase control arguments
        phase_group = parser.add_argument_group("Phase Control", "Selective benchmark execution")

        phase_group.add_argument(
            "--phase",
            type=str,
            default="power",
            help="Benchmark phases to run (comma-separated): generate,load,warmup,power,throughput,maintenance",
        )

    @classmethod
    def add_output_arguments(cls, parser: argparse.ArgumentParser) -> None:
        """Add output and result-related arguments.

        Args:
            parser: ArgumentParser to add output arguments to
        """
        output_group = parser.add_argument_group("Output Options", "Result reporting and data export")

        output_group.add_argument(
            "--save-results",
            type=str,
            metavar="FILE",
            help="Save detailed results to JSON file",
        )
        output_group.add_argument(
            "--export-format",
            choices=["json", "csv", "html"],
            action="append",
            help="Export result formats",
        )
        output_group.add_argument(
            "--dry-run",
            type=str,
            metavar="DIR",
            help="Show execution plan without running (output to directory)",
        )

    @classmethod
    def add_platform_arguments(cls, parser: argparse.ArgumentParser, platform: str) -> None:
        """Add platform-specific arguments.

        Args:
            parser: ArgumentParser to add platform arguments to
            platform: Platform name (e.g., 'databricks', 'duckdb', 'clickhouse')
        """
        platform_title = platform.title()

        if platform.lower() == "databricks":
            cls._add_databricks_arguments(parser, platform_title)
        elif platform.lower() == "duckdb":
            cls._add_duckdb_arguments(parser, platform_title)
        elif platform.lower() == "clickhouse":
            cls._add_clickhouse_arguments(parser, platform_title)

    @classmethod
    def _add_databricks_arguments(cls, parser: argparse.ArgumentParser, platform_title: str) -> None:
        """Add Databricks-specific arguments."""
        databricks_group = parser.add_argument_group(
            f"{platform_title} Configuration",
            "Platform-specific settings and authentication",
        )

        databricks_group.add_argument("--server-hostname", type=str, help="Databricks server hostname")
        databricks_group.add_argument("--http-path", type=str, help="SQL warehouse HTTP path")
        databricks_group.add_argument("--access-token", type=str, help="Databricks access token")
        databricks_group.add_argument(
            "--catalog",
            type=str,
            default="main",
            help="Unity Catalog name (default: main)",
        )
        databricks_group.add_argument(
            "--schema",
            type=str,
            default="benchbox",
            help="Database schema name (default: benchbox)",
        )
        databricks_group.add_argument(
            "--create-catalog",
            action="store_true",
            help="Auto-create catalog if it doesn't exist",
        )

        # Delta Lake optimizations
        delta_group = parser.add_argument_group("Delta Lake Optimizations", "Databricks-specific performance tuning")

        delta_group.add_argument(
            "--enable-delta-optimization",
            action="store_true",
            help="Enable Delta table optimizations",
        )
        delta_group.add_argument(
            "--auto-optimize",
            action="store_true",
            help="Enable auto-optimization for Delta tables",
        )
        delta_group.add_argument(
            "--auto-compact",
            action="store_true",
            help="Enable auto-compaction for Delta tables",
        )
        delta_group.add_argument(
            "--cluster-size",
            choices=["XS", "Small", "Medium", "Large", "XL"],
            help="Databricks cluster size",
        )

    @classmethod
    def _add_duckdb_arguments(cls, parser: argparse.ArgumentParser, platform_title: str) -> None:
        """Add DuckDB-specific arguments."""
        duckdb_group = parser.add_argument_group(
            f"{platform_title} Configuration",
            "Platform-specific settings and optimizations",
        )

        duckdb_group.add_argument(
            "--memory-limit",
            type=str,
            help="DuckDB memory limit (e.g., '4GB', '512MB')",
        )
        duckdb_group.add_argument("--threads", type=int, help="Number of threads for query execution")
        duckdb_group.add_argument("--temp-directory", type=str, help="Temporary directory for spill files")

    @classmethod
    def _add_clickhouse_arguments(cls, parser: argparse.ArgumentParser, platform_title: str) -> None:
        """Add ClickHouse-specific arguments."""
        clickhouse_group = parser.add_argument_group(
            f"{platform_title} Configuration",
            "Platform-specific settings and connection",
        )

        # Mode selection (server or embedded)
        clickhouse_group.add_argument(
            "--mode",
            type=str,
            choices=["server", "embedded"],
            default="server",
            help="ClickHouse mode: 'server' for remote/local server, 'local' for in-process (default: server)",
        )

        # Server mode arguments
        server_group = parser.add_argument_group(
            f"{platform_title} Server Mode",
            "Arguments for server mode (when --mode=server)",
        )
        server_group.add_argument(
            "--host",
            type=str,
            default="localhost",
            help="ClickHouse server host (default: localhost)",
        )
        server_group.add_argument(
            "--port",
            type=int,
            default=9000,
            help="ClickHouse server port (default: 9000)",
        )
        server_group.add_argument(
            "--user",
            type=str,
            default="default",
            help="ClickHouse username (default: default)",
        )
        server_group.add_argument("--password", type=str, help="ClickHouse password")
        server_group.add_argument("--secure", action="store_true", help="Use secure connection (TLS)")

        # Embedded mode arguments
        embedded_group = parser.add_argument_group(
            f"{platform_title} Embedded Mode",
            "Arguments for embedded mode (when --mode=embedded)",
        )
        embedded_group.add_argument(
            "--data-path",
            type=str,
            help="Data path for embedded mode file operations (optional)",
        )

    @classmethod
    def parse_and_validate_args(cls, parser: argparse.ArgumentParser) -> argparse.Namespace:
        """Parse arguments and apply validation rules.

        Args:
            parser: Configured ArgumentParser

        Returns:
            Parsed and validated arguments

        Raises:
            SystemExit: If validation fails
        """
        args = parser.parse_args()

        # Mutual exclusion validation
        exclusive_groups = [
            (["power_only", "throughput_only", "load_only"], "Phase control options"),
            (["verbose", "quiet"], "Output verbosity options"),
        ]

        for group, description in exclusive_groups:
            active_options = [opt for opt in group if getattr(args, opt, False)]
            if len(active_options) > 1:
                parser.error(
                    f"{description} are mutually exclusive: {', '.join(['--' + opt.replace('_', '-') for opt in active_options])}"
                )

        return args
